@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,20 +15,18 @@ from gi.repository import Gtk, Gdk, GLib, Gio, Gst
 from datetime import datetime
 from gettext import gettext as _
 
-from lollypop.define import App, ArtSize, ArtBehaviour, AdaptiveSize
+from lollypop.define import App, ArtSize, ArtBehaviour
+from lollypop.define import MARGIN_BIG
 from lollypop.widgets_player_progress import ProgressPlayerWidget
 from lollypop.widgets_player_buttons import ButtonsPlayerWidget
 from lollypop.widgets_player_artwork import ArtworkPlayerWidget
 from lollypop.widgets_player_label import LabelPlayerWidget
-from lollypop.objects_radio import Radio
 from lollypop.container import Container
-from lollypop.adaptive import AdaptiveWindow
 from lollypop.logger import Logger
-from lollypop.utils import emit_signal
 from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
+class FullScreen(Gtk.Window, SignalsHelper):
     """
         Show a fullscreen window showing current track context
     """
@@ -39,7 +37,6 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
             Init window
         """
         Gtk.Window.__init__(self)
-        AdaptiveWindow.__init__(self)
         return [
                 (App().player, "current-changed", "_on_current_changed")
         ]
@@ -56,18 +53,9 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__signal1_id = self.__signal2_id = None
         self.__background_id = None
         self.set_decorated(False)
-        # Calculate cover size
-        screen = Gdk.Screen.get_default()
-        monitor = screen.get_monitor_at_window(App().main_window.get_window())
-        geometry = screen.get_monitor_geometry(monitor)
-        art_size_fs = ArtSize.FULLSCREEN / self.get_scale_factor()
-        font_size_fs = 40 / self.get_scale_factor()
-        if geometry.width > geometry.height:
-            art_size = int(art_size_fs * geometry.height / 1080)
-            font_size = int(font_size_fs * geometry.height / 1080)
-        else:
-            art_size = int(art_size_fs * geometry.width / 1920)
-            font_size = int(font_size_fs * geometry.width / 1920)
+        scale_factor = self.get_scale_factor()
+        art_size = ArtSize.FULLSCREEN / scale_factor
+        font_size = 30 / scale_factor
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/FullScreen.ui")
         builder.connect_signals(self)
@@ -80,7 +68,8 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__buttons_widget.set_size_request(500, -1)
         self.__buttons_widget.set_property("valign", Gtk.Align.CENTER)
         self.__buttons_widget.set_property("halign", Gtk.Align.CENTER)
-        self.__artwork_widget = ArtworkPlayerWidget()
+        self.__artwork_widget = ArtworkPlayerWidget(
+            ArtBehaviour.CACHE | ArtBehaviour.CROP_SQUARE)
         self.__artwork_widget.show()
         self.__artwork_widget.set_vexpand(True)
         self.__artwork_widget.set_art_size(art_size, art_size)
@@ -93,8 +82,6 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__label_widget.set_justify(Gtk.Justification.CENTER)
         eventbox = Gtk.EventBox.new()
         eventbox.show()
-        eventbox.set_property("valign", Gtk.Align.END)
-        eventbox.set_property("halign", Gtk.Align.END)
         eventbox.connect("button-release-event",
                          self.__on_image_button_release_event)
         eventbox.connect("realize", self.__on_image_realize)
@@ -108,9 +95,11 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
             self.__overlay_grid.attach(self.__label_widget, 0, 2, 2, 1)
             self.__overlay_grid.attach(self.__progress_widget, 0, 3, 2, 1)
             self.__overlay_grid.attach(eventbox, 2, 2, 1, 3)
-            self.__progress_widget.set_margin_start(50)
-            eventbox.set_margin_end(50)
-            self.__label_widget.set_margin_start(50)
+            eventbox.set_margin_end(MARGIN_BIG)
+            eventbox.set_property("valign", Gtk.Align.END)
+            eventbox.set_property("halign", Gtk.Align.END)
+            self.__artwork_widget.set_margin_end(MARGIN_BIG)
+            self.__artwork_widget.set_margin_bottom(MARGIN_BIG)
             self.__label_widget.set_property("valign", Gtk.Align.END)
             self.__artwork_widget.set_property("valign", Gtk.Align.END)
         else:
@@ -146,6 +135,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__background_artwork = builder.get_object("background_artwork")
         self.__container = Container()
         self.__container.show()
+        self.__container.setup()
         self.__sidebar = Gtk.Grid()
         self.__sidebar.set_size_request(400, -1)
         self.__sidebar.set_orientation(Gtk.Orientation.VERTICAL)
@@ -169,6 +159,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__buttons_widget.update()
         self.__label_widget.update()
         self.__progress_widget.update()
+        self.__update_progress_visibility()
         self.__setup_artwork_type()
         if self.__timeout_id is None:
             try:
@@ -185,7 +176,6 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         screen = Gdk.Screen.get_default()
         monitor = screen.get_monitor_at_window(App().main_window.get_window())
         self.fullscreen_on_monitor(screen, monitor)
-        self.__container.setup_lists()
         # Disable screensaver (idle)
         App().inhibitor.manual_inhibit(
                 Gtk.ApplicationInhibitFlags.IDLE |
@@ -205,18 +195,14 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
     def miniplayer(self):
         return App().main_window.miniplayer
 
+    @property
+    def folded(self):
+        return False
+
     @property
     def toolbar(self):
         return App().main_window.toolbar
 
-    @property
-    def is_adaptive(self):
-        return True
-
-    @property
-    def adaptive_size(self):
-        return AdaptiveSize.PHONE
-
     @property
     def container(self):
         """
@@ -246,7 +232,6 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
                                                   Gtk.IconSize.BUTTON)
         else:
             self.__revealer.set_reveal_child(True)
-            emit_signal(self, "adaptive-changed", True)
             button.get_image().set_from_icon_name("pan-end-symbolic",
                                                   Gtk.IconSize.BUTTON)
 
@@ -256,6 +241,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
             @param player as Player
         """
         self.__update_background()
+        self.__update_progress_visibility()
 
 #######################
 # PRIVATE             #
@@ -277,6 +263,15 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
         self.__artwork_widget.set_behaviour(behaviour)
         self.__artwork_widget.update()
 
+    def __update_progress_visibility(self):
+        """
+            Update progress bar visibility
+        """
+        if App().player.current_track.id is not None:
+            self.__progress_widget.show()
+        else:
+            self.__progress_widget.hide()
+
     def __update_background(self, album_artwork=False):
         """
             Update window background
@@ -290,22 +285,10 @@ class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
                 return
             behaviour = App().settings.get_value("fullscreen-type").get_int32()
             behaviour |= (ArtBehaviour.CROP |
-                          ArtBehaviour.CACHE |
                           ArtBehaviour.DARKER)
             # We don't want this for background, stored for album cover
             behaviour &= ~ArtBehaviour.ROUNDED
-            if isinstance(App().player.current_track, Radio):
-                if self.__background_id == App().player.current_track.name:
-                    return
-                App().art_helper.set_radio_artwork(
-                                    App().player.current_track.name,
-                                    allocation.width,
-                                    allocation.height,
-                                    self.get_scale_factor(),
-                                    behaviour | ArtBehaviour.BLUR_MAX,
-                                    self.__on_artwork,
-                                    False)
-            elif not album_artwork and\
+            if not album_artwork and\
                     App().settings.get_value("artist-artwork"):
                 if App().player.current_track.album.artists:
                     artist = App().player.current_track.album.artists[0]
