@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -59,7 +59,7 @@ class ArtHelper(GObject.Object):
             @param effect as ArtBehaviour
             @param callback as function
         """
-        App().task_helper.run(App().art.get_album_artwork,
+        App().task_helper.run(App().album_art.get,
                               album,
                               width,
                               height,
@@ -73,31 +73,6 @@ class ArtHelper(GObject.Object):
                                         callback,
                                         *args))
 
-    def set_radio_artwork(self, radio, width, height, scale_factor,
-                          effect, callback, *args):
-        """
-            Set artwork for album id
-            @param radio as str
-            @param width as int
-            @param height as int
-            @param scale_factor as int
-            @param effect as ArtBehaviour
-            @param callback as function
-        """
-        App().task_helper.run(App().art.get_radio_artwork,
-                              radio,
-                              width,
-                              height,
-                              scale_factor,
-                              effect,
-                              callback=(self._on_get_artwork_pixbuf,
-                                        width,
-                                        height,
-                                        scale_factor,
-                                        effect,
-                                        callback,
-                                        *args))
-
     def set_artist_artwork(self, name, width, height, scale_factor,
                            effect, callback, *args):
         """
@@ -109,7 +84,7 @@ class ArtHelper(GObject.Object):
             @param effect as ArtBehaviour
             @param callback as function
         """
-        App().task_helper.run(App().art.get_artist_artwork,
+        App().task_helper.run(App().artist_art.get,
                               name,
                               width,
                               height,
@@ -146,7 +121,6 @@ class ArtHelper(GObject.Object):
             else:
                 surface = Gdk.cairo_surface_create_from_pixbuf(
                         pixbuf, scale_factor, None)
-            del pixbuf
         App().task_helper.run(self.__surface_effects, surface, width, height,
                               scale_factor, effect, callback, *args)
 
