@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -13,7 +13,6 @@
 from gi.repository import Gio, GLib
 
 from lollypop.logger import Logger
-from lollypop.objects_radio import Radio
 from lollypop.helper_task import TaskHelper
 from lollypop.utils import escape, get_network_available
 from lollypop.utils_file import create_dir
@@ -55,7 +54,10 @@ class LyricsHelper:
             if info is not None:
                 tags = info.get_tags()
                 for (lyrics, timestamp) in tagreader.get_synced_lyrics(tags):
-                    self.__timestamps[timestamp] = lyrics
+                    if timestamp in self.__timestamps.keys():
+                        self.__timestamps[timestamp] += "\n%s" % lyrics
+                    else:
+                        self.__timestamps[timestamp] = lyrics
 
     def get_lyrics_for_timestamp(self, timestamp):
         """
@@ -90,10 +92,14 @@ class LyricsHelper:
         """
         self.__cancellable = Gio.Cancellable.new()
         methods = []
+        if get_network_available("BING"):
+            methods.append(self.__download_bing_lyrics)
         if get_network_available("WIKIA"):
             methods.append(self.__download_wikia_lyrics)
         if get_network_available("GENIUS"):
             methods.append(self.__download_genius_lyrics)
+        if get_network_available("METROLYRICS"):
+            methods.append(self.__download_metro_lyrics)
         if methods:
             self.__get_lyrics_from_web(track, methods, callback, *args)
         else:
@@ -146,7 +152,10 @@ class LyricsHelper:
                         str_timestamp = line.split("]")[0].split("[")[1]
                         timestamp = self.__str_to_timestamp(str_timestamp)
                         lyrics = " ".join(line.split("]")[1:])
-                        self.__timestamps[timestamp] = lyrics
+                        if timestamp in self.__timestamps.keys():
+                            self.__timestamps[timestamp] += "\n%s" % lyrics
+                        else:
+                            self.__timestamps[timestamp] = lyrics
                     except:
                         continue
         except Exception as e:
@@ -173,13 +182,7 @@ class LyricsHelper:
             @return str
         """
         # Update lyrics
-        title = ""
-        if isinstance(track, Radio):
-            split = " ".join(track.artists).split(" - ")
-            if len(split) > 1:
-                title = split[1]
-        else:
-            title = track.name
+        title = track.name
         if escape:
             return GLib.uri_escape_string(title, None, False)
         else:
@@ -192,22 +195,59 @@ class LyricsHelper:
             @param escape as bool
             @return str
         """
-        # Update lyrics
-        artist = ""
-        if isinstance(track, Radio):
-            split = " ".join(track.artists).split(" - ")
-            if len(split) > 0:
-                artist = split[0]
-        else:
-            if track.artists:
-                artist = track.artists[0]
-            elif track.album_artists:
-                artist = track.album_artists[0]
+        if track.artists:
+            artist = track.artists[0]
+        elif track.album_artists:
+            artist = track.album_artists[0]
         if escape:
             return GLib.uri_escape_string(artist, None, False)
         else:
             return artist
 
+    def __download_bing_lyrics(self, track, methods, callback, *args):
+        """
+            Downloas lyrics from metro lyrics
+            @param track as Track
+            @param methods as []
+            @param callback as function
+        """
+        title = self.__get_title(track, False)
+        artist = self.__get_artist(track, False).lower()
+        string = "%s %s" % (artist, title)
+        uri = "https://www.bing.com/search?q=%s" % string
+        helper = TaskHelper()
+        helper.load_uri_content(uri,
+                                self.__cancellable,
+                                self.__on_lyrics_downloaded,
+                                "b_heroLyrics",
+                                "\n",
+                                track,
+                                methods,
+                                callback,
+                                *args)
+
+    def __download_metro_lyrics(self, track, methods, callback, *args):
+        """
+            Downloas lyrics from metro lyrics
+            @param track as Track
+            @param methods as []
+            @param callback as function
+        """
+        title = self.__get_title(track, False)
+        artist = self.__get_artist(track, False).lower()
+        string = "%s-lyrics-%s" % (title, artist)
+        uri = "https://www.metrolyrics.com/%s.html" % string.replace(" ", "-")
+        helper = TaskHelper()
+        helper.load_uri_content(uri,
+                                self.__cancellable,
+                                self.__on_lyrics_downloaded,
+                                "lyrics-body",
+                                "",
+                                track,
+                                methods,
+                                callback,
+                                *args)
+
     def __download_wikia_lyrics(self, track, methods, callback, *args):
         """
             Downloas lyrics from wikia
@@ -215,9 +255,10 @@ class LyricsHelper:
             @param methods as []
             @param callback as function
         """
-        title = self.__get_title(track, True)
-        artist = self.__get_artist(track, True)
-        uri = "https://lyrics.wikia.com/wiki/%s:%s" % (artist, title)
+        title = self.__get_title(track, False)
+        artist = self.__get_artist(track, False).lower()
+        string = "%s:%s" % (artist, title)
+        uri = "https://lyrics.wikia.com/wiki/%s" % string.replace(" ", "_")
         helper = TaskHelper()
         helper.load_uri_content(uri,
                                 self.__cancellable,
@@ -238,7 +279,9 @@ class LyricsHelper:
         """
         title = self.__get_title(track, False)
         artist = self.__get_artist(track, False)
-        string = escape("%s %s" % (artist, title)).replace(".", "")
+        string = escape("%s %s" % (artist, title),
+                        ignore=["_", "-", " ", ".", "/"]).replace(".", "")
+        string = string.replace("/", " ")
         uri = "https://genius.com/%s-lyrics" % string.replace(" ", "-")
         helper = TaskHelper()
         helper.load_uri_content(uri,
@@ -264,6 +307,7 @@ class LyricsHelper:
             @param methods as []
             @param callback as function
         """
+        Logger.debug("LyricsHelper::__on_lyrics_downloaded(): %s", uri)
         if status:
             try:
                 from bs4 import BeautifulSoup
@@ -273,5 +317,5 @@ class LyricsHelper:
                 callback(lyrics, *args)
                 return
             except Exception as e:
-                Logger.warning("LyricsView::__on_lyrics_downloaded(): %s", e)
+                Logger.warning("LyricsHelper::__on_lyrics_downloaded(): %s", e)
         self.__get_lyrics_from_web(track, methods, callback, *args)
