@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,11 +12,10 @@
 
 from gi.repository import Gio, Gtk, GLib
 
-from pickle import load, dump
 from gettext import gettext as _
 
-from lollypop.define import StorageType, MARGIN_SMALL, App, CACHE_PATH
-from lollypop.define import ViewType
+from lollypop.define import StorageType, MARGIN_SMALL, App
+from lollypop.define import ViewType, Type
 from lollypop.menu_playlists import PlaylistsMenu
 from lollypop.menu_actions import ActionsMenu
 from lollypop.menu_playback import TrackPlaybackMenu, AlbumPlaybackMenu
@@ -38,14 +37,14 @@ class AlbumMenu(Gio.Menu):
             @param view_type as ViewType
         """
         Gio.Menu.__init__(self)
-        if view_type & ViewType.ADAPTIVE:
+        if App().window.folded:
             from lollypop.menu_header import AlbumMenuHeader
             self.append_item(AlbumMenuHeader(album))
         self.append_section(_("Playback"),
                             AlbumPlaybackMenu(album, view_type))
-        from lollypop.menu_artist import ArtistAlbumsMenu
-        menu = ArtistAlbumsMenu(album.artist_ids[0], album.storage_type)
-        if menu.get_n_items() != 0:
+        if view_type & ViewType.ALBUM:
+            from lollypop.menu_artist import ArtistAlbumsMenu
+            menu = ArtistAlbumsMenu(album.artist_ids[0], album.storage_type)
             self.append_section(_("Artist"), menu)
         section = Gio.Menu()
         if album.storage_type & (StorageType.COLLECTION | StorageType.SAVED):
@@ -54,7 +53,21 @@ class AlbumMenu(Gio.Menu):
             section.append_submenu(_("Devices"), SyncAlbumsMenu([album]))
         if section.get_n_items() != 0:
             self.append_section(_("Add to"), section)
-        self.append_section(_("Others"), ActionsMenu(album))
+        actions_menu = ActionsMenu(album)
+        # Allow user to show album if not in album view
+        if not view_type & ViewType.ALBUM:
+            show_album_action = Gio.SimpleAction(name="show_album_action")
+            App().add_action(show_album_action)
+            show_album_action.connect(
+                "activate",
+                lambda x, y:
+                App().window.container.show_view(
+                   [Type.ALBUM], album, storage_type))
+            menu_item = Gio.MenuItem.new(_("Show album"),
+                                         "app.show_album_action")
+            menu_item.set_attribute_value("close", GLib.Variant("b", True))
+            actions_menu.prepend_item(menu_item)
+        self.append_section(_("Others"), actions_menu)
 
 
 class AlbumsMenu(Gio.Menu):
@@ -70,7 +83,7 @@ class AlbumsMenu(Gio.Menu):
             @param view_type as ViewType
         """
         Gio.Menu.__init__(self)
-        if view_type & ViewType.ADAPTIVE:
+        if App().window.folded:
             from lollypop.menu_header import MenuHeader
             self.append_item(MenuHeader(title,
                                         "media-optical-cd-audio-symbolic"))
@@ -90,7 +103,7 @@ class TrackMenu(Gio.Menu):
 
         """
         Gio.Menu.__init__(self)
-        if view_type & ViewType.ADAPTIVE:
+        if App().window.folded:
             from lollypop.menu_header import AlbumMenuHeader
             self.append_item(AlbumMenuHeader(track))
         self.append_section(_("Playback"), TrackPlaybackMenu(track, view_type))
@@ -135,13 +148,13 @@ class TrackMenuExt(Gtk.Grid):
         hgrid = Gtk.Grid()
         rating = RatingWidget(track)
         rating.set_property("halign", Gtk.Align.START)
-        if App().window.is_adaptive:
+        if App().window.folded:
             rating.set_icon_size(Gtk.IconSize.LARGE_TOOLBAR)
         rating.show()
 
         loved = LovedWidget(track)
         loved.set_property("halign", Gtk.Align.START)
-        if App().window.is_adaptive:
+        if App().window.folded:
             loved.set_icon_size(Gtk.IconSize.LARGE_TOOLBAR)
         loved.show()
 
@@ -151,12 +164,9 @@ class TrackMenuExt(Gtk.Grid):
         hgrid.add(rating)
         hgrid.show()
 
-        if not track.storage_type & StorageType.COLLECTION:
-            try:
-                escaped = GLib.uri_escape_string(track.uri, None, True)
-                uri = load(open("%s/web_%s" % (CACHE_PATH, escaped), "rb"))
-            except:
-                uri = ""
+        if track.is_web:
+            from lollypop.helper_web import WebHelper
+            helper = WebHelper(track, None)
             edit = Gtk.Entry()
             edit.set_placeholder_text(_("YouTube page address"))
             edit.set_margin_top(MARGIN_SMALL)
@@ -164,7 +174,8 @@ class TrackMenuExt(Gtk.Grid):
             edit.set_margin_end(MARGIN_SMALL)
             edit.set_margin_bottom(MARGIN_SMALL)
             edit.set_property("hexpand", True)
-            edit.set_text(uri)
+            if helper.uri is not None:
+                edit.set_text(helper.uri)
             edit.connect("changed", self.__on_edit_changed, track)
             edit.show()
             self.add(edit)
@@ -174,6 +185,10 @@ class TrackMenuExt(Gtk.Grid):
         self.add(separator)
         self.add(hgrid)
 
+    @property
+    def section(self):
+        return None
+
     @property
     def submenu_name(self):
         return None
@@ -190,12 +205,8 @@ class TrackMenuExt(Gtk.Grid):
         from urllib.parse import urlparse
         uri = edit.get_text()
         parsed = urlparse(uri)
-        escaped = GLib.uri_escape_string(track.uri, None, True)
         if parsed.scheme not in ["http", "https"]:
             return
-        try:
-            # CACHE URI
-            with open("%s/web_%s" % (CACHE_PATH, escaped), "wb") as f:
-                dump(uri, f)
-        except:
-            pass
+        from lollypop.helper_web import WebHelper
+        helper = WebHelper(track, None)
+        helper.save(uri)
