@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -14,9 +14,10 @@ from gi.repository import Gio, GLib
 
 from gettext import gettext as _
 
-from lollypop.define import App, ViewType, Type
+from lollypop.define import App, ViewType, Type, LovedFlags
 from lollypop.utils_album import tracks_to_albums
 from lollypop.utils import get_default_storage_type, emit_signal
+from lollypop.utils import get_network_available
 from lollypop.objects_track import Track
 from lollypop.objects_album import Album
 
@@ -43,6 +44,21 @@ class PlaybackMenu(Gio.Menu):
 #######################
 # PROTECTED           #
 #######################
+    def _set_radio_action(self, artist_ids):
+        """
+            Set radio action
+            @param artist_ids as [int]
+        """
+        radio_action = Gio.SimpleAction(name="radio_action_collection")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Play a radio"),
+                                     "app.radio_action_collection")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        self.append_item(menu_item)
+
     def _set_playback_actions(self):
         """
             Setup playback actions
@@ -65,6 +81,117 @@ class PlaybackMenu(Gio.Menu):
         menu_item.set_attribute_value("close", GLib.Variant("b", True))
         self.append_item(menu_item)
 
+#######################
+# PRIVATE             #
+#######################
+    def __on_radio_action_activate(self, action, variant, artist_ids):
+        """
+            Play a radio from storage type
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param artist_ids as [int]
+        """
+        App().player.play_radio_from_collection(artist_ids)
+
+
+class RadioPlaybackMenu(Gio.Menu):
+    """
+        Allow to play a radio from artist ids
+    """
+
+    def __init__(self, artist_ids):
+        """
+            Init menu
+            @param artist_ids as [int]
+        """
+        Gio.Menu.__init__(self)
+        section = Gio.Menu()
+        radio_action = Gio.SimpleAction(name="radio_action_collection")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Related tracks"),
+                                     "app.radio_action_collection")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_loved")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Loved tracks"),
+                                     "app.radio_action_loved")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_populars")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Popular tracks"),
+                                     "app.radio_action_populars")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        self.append_section(_("From collection"), section)
+        section = Gio.Menu()
+        radio_action = Gio.SimpleAction(name="radio_action_deezer")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("DEEZER"))
+        menu_item = Gio.MenuItem.new(_("Deezer"),
+                                     "app.radio_action_deezer")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_lastfm")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("LASTFM"))
+        menu_item = Gio.MenuItem.new(_("Last.fm"),
+                                     "app.radio_action_lastfm")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_spotify")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("SPOTIFY"))
+        menu_item = Gio.MenuItem.new(_("Spotify"),
+                                     "app.radio_action_spotify")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        self.append_section(_("From the Web"), section)
+
+#######################
+# PRIVATE             #
+#######################
+    def __on_radio_action_activate(self, action, variant, artist_ids):
+        """
+            Play a radio from storage type
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param artist_ids as [int]
+        """
+        def play_radio():
+            if action.get_name() == "radio_action_collection":
+                App().player.play_radio_from_collection(artist_ids)
+            elif action.get_name() == "radio_action_spotify":
+                App().player.play_radio_from_spotify(artist_ids)
+            elif action.get_name() == "radio_action_lastfm":
+                App().player.play_radio_from_lastfm(artist_ids)
+            elif action.get_name() == "radio_action_deezer":
+                App().player.play_radio_from_deezer(artist_ids)
+            elif action.get_name() == "radio_action_loved":
+                App().player.play_radio_from_loved(artist_ids)
+            elif action.get_name() == "radio_action_populars":
+                App().player.play_radio_from_populars(artist_ids)
+        App().task_helper.run(play_radio)
+
 
 class PlaylistPlaybackMenu(Gio.Menu):
     """
@@ -79,21 +206,26 @@ class PlaylistPlaybackMenu(Gio.Menu):
         Gio.Menu.__init__(self)
         play_action = Gio.SimpleAction(name="playlist_play_action")
         App().add_action(play_action)
-        play_action.connect("activate", self.__play, playlist_id)
+        play_action.connect("activate", self.__on_play_action, playlist_id)
         menu_item = Gio.MenuItem.new(_("Play this playlist"),
                                      "app.playlist_play_action")
         menu_item.set_attribute_value("close", GLib.Variant("b", True))
         self.append_item(menu_item)
+        add_action = Gio.SimpleAction(name="playlist_add_action")
+        App().add_action(add_action)
+        add_action.connect("activate", self.__on_add_action, playlist_id)
+        menu_item = Gio.MenuItem.new(_("Add this playlist"),
+                                     "app.playlist_add_action")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        self.append_item(menu_item)
 
 #######################
 # PRIVATE             #
 #######################
-    def __play(self, action, variant, playlist_id):
+    def __get_albums(self, playlist_id):
         """
-            Play albums
-            @param Gio.SimpleAction
-            @param GLib.Variant
-            @param playlist_id as int
+            Get albums for playlist_id
+            @parma playlist_id as int
         """
         if App().playlists.get_smart(playlist_id):
             request = App().playlists.get_smart_sql(playlist_id)
@@ -108,7 +240,25 @@ class PlaylistPlaybackMenu(Gio.Menu):
         else:
             tracks = App().playlists.get_tracks(playlist_id)
             albums = tracks_to_albums(tracks)
-        App().player.play_albums(albums)
+        return albums
+
+    def __on_play_action(self, action, variant, playlist_id):
+        """
+            Play albums
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param playlist_id as int
+        """
+        App().player.play_albums(self.__get_albums(playlist_id))
+
+    def __on_add_action(self, action, variant, playlist_id):
+        """
+            Add albums to playback
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param playlist_id as int
+        """
+        App().player.add_albums(self.__get_albums(playlist_id))
 
 
 class ArtistPlaybackMenu(PlaybackMenu):
@@ -127,12 +277,20 @@ class ArtistPlaybackMenu(PlaybackMenu):
         self.__storage_type = storage_type
         play_action = Gio.SimpleAction(name="artist_play_action")
         App().add_action(play_action)
-        play_action.connect("activate", self.__play)
+        play_action.connect("activate", self.__on_play_action_activate)
         menu_item = Gio.MenuItem.new(_("Play this artist"),
                                      "app.artist_play_action")
         menu_item.set_attribute_value("close", GLib.Variant("b", True))
         self.append_item(menu_item)
         self._set_playback_actions()
+        self.__set_skipped_action()
+        if get_network_available("SPOTIFY") or\
+                get_network_available("LASTFM") or\
+                get_network_available("DEEZER"):
+            submenu = RadioPlaybackMenu([artist_id])
+            self.append_submenu(_("Play a radio"), submenu)
+        else:
+            self._set_radio_action([artist_id])
 
     @property
     def in_player(self):
@@ -140,8 +298,8 @@ class ArtistPlaybackMenu(PlaybackMenu):
             True if current object in player
             return bool
         """
-        album_ids = App().albums.get_ids([self.__artist_id], [],
-                                         self.__storage_type)
+        album_ids = App().albums.get_ids([], [self.__artist_id],
+                                         self.__storage_type, False)
         return set(App().player.album_ids) & set(album_ids) == set(album_ids)
 
 #######################
@@ -168,7 +326,43 @@ class ArtistPlaybackMenu(PlaybackMenu):
 #######################
 # PRIVATE             #
 #######################
-    def __play(self, action, variant):
+    def __set_skipped_action(self):
+        """
+            Set skipped action
+        """
+        album_ids = App().albums.get_ids([], [self.__artist_id],
+                                         self.__storage_type, False)
+        skipped = True
+        for album_id in album_ids:
+            album = Album(album_id)
+            if not album.loved & LovedFlags.SKIPPED:
+                skipped = False
+                break
+        action = Gio.SimpleAction.new_stateful(
+                "skip-artist",
+                None,
+                GLib.Variant.new_boolean(skipped))
+        App().add_action(action)
+        action.connect("change-state", self.__on_loved_change_state)
+        self.append(_("Ignored"), "app.skip-artist")
+
+    def __on_loved_change_state(self, action, state):
+        """
+            Update Skipped state
+            @param action as Gio.SimpleAction
+            @param state as bool
+        """
+        action.set_state(state)
+        album_ids = App().albums.get_ids([], [self.__artist_id],
+                                         self.__storage_type, False)
+        for album_id in album_ids:
+            album = Album(album_id)
+            if state:
+                album.set_loved(album.loved | LovedFlags.SKIPPED)
+            else:
+                album.set_loved(album.loved & ~LovedFlags.SKIPPED)
+
+    def __on_play_action_activate(self, action, variant):
         """
             Play albums
             @param Gio.SimpleAction
@@ -227,6 +421,8 @@ class GenrePlaybackMenu(PlaybackMenu):
             @param GLib.Variant
         """
         album_ids = self.__get_album_ids()
+        if App().player.current_track.album.id in album_ids:
+            App().player.skip_album()
         App().player.remove_album_by_ids(album_ids)
 
 #######################
@@ -240,11 +436,10 @@ class GenrePlaybackMenu(PlaybackMenu):
         storage_type = get_default_storage_type()
         album_ids = App().albums.get_compilation_ids([self.__genre_id],
                                                      storage_type,
-                                                     True)
-        album_ids += App().albums.get_ids([],
-                                          [self.__genre_id],
+                                                     False)
+        album_ids += App().albums.get_ids([self.__genre_id], [],
                                           storage_type,
-                                          True)
+                                          False)
         return album_ids
 
     def __play(self, action, variant):
@@ -307,6 +502,8 @@ class DecadePlaybackMenu(PlaybackMenu):
             @param GLib.Variant
         """
         album_ids = self.__get_album_ids()
+        if App().player.current_track.album.id in album_ids:
+            App().player.skip_album()
         App().player.remove_album_by_ids(album_ids)
 
 #######################
@@ -320,10 +517,10 @@ class DecadePlaybackMenu(PlaybackMenu):
         storage_type = get_default_storage_type()
         album_ids = []
         for year in self.__years:
-            album_ids += App().albums.get_compilations_for_year(
-                year, storage_type)
-            album_ids += App().albums.get_albums_for_year(
-                year, storage_type)
+            album_ids += App().albums.get_compilation_ids_for_year(
+                year, storage_type, False)
+            album_ids += App().albums.get_ids_for_year(
+                year, storage_type, False)
         return album_ids
 
     def __play(self, action, variant):
@@ -342,21 +539,38 @@ class AlbumPlaybackMenu(PlaybackMenu):
         Contextual menu for an album
     """
 
-    def __init__(self, album):
+    def __init__(self, album, view_type):
         """
             Init album menu
             @param album as Album
+            @param view_type as ViewType
         """
         PlaybackMenu.__init__(self)
         self.__album = album
-        play_action = Gio.SimpleAction(name="album_play_action")
-        App().add_action(play_action)
-        play_action.connect("activate", self.__play)
-        menu_item = Gio.MenuItem.new(_("Play this album"),
-                                     "app.album_play_action")
-        menu_item.set_attribute_value("close", GLib.Variant("b", True))
-        self.append_item(menu_item)
-        self._set_playback_actions()
+        if not view_type & ViewType.BANNER:
+            play_action = Gio.SimpleAction(name="play_action")
+            App().add_action(play_action)
+            play_action.connect("activate", self.__on_play_action_activate)
+            menu_item = Gio.MenuItem.new(_("Play this album"),
+                                         "app.play_action")
+            menu_item.set_attribute_value("close", GLib.Variant("b", True))
+            self.append_item(menu_item)
+            self._set_playback_actions()
+        action = Gio.SimpleAction.new_stateful(
+                "skip-album",
+                None,
+                GLib.Variant.new_boolean(
+                    self.__album.loved & LovedFlags.SKIPPED))
+        App().add_action(action)
+        action.connect("change-state", self.__on_loved_change_state)
+        self.append(_("Ignored"), "app.skip-album")
+        if get_network_available("SPOTIFY") or\
+                get_network_available("LASTFM") or\
+                get_network_available("DEEZER"):
+            submenu = RadioPlaybackMenu(album.artist_ids)
+            self.append_submenu(_("Play a radio"), submenu)
+        else:
+            self._set_radio_action(album.artist_ids)
 
     @property
     def in_player(self):
@@ -383,18 +597,32 @@ class AlbumPlaybackMenu(PlaybackMenu):
             @param Gio.SimpleAction
             @param GLib.Variant
         """
+        if App().player.current_track.album.id == self.__album.id:
+            App().player.skip_album()
         App().player.remove_album_by_id(self.__album.id)
 
 #######################
 # PRIVATE             #
 #######################
-    def __play(self, action, variant):
+    def __on_loved_change_state(self, action, state):
+        """
+            Update Skipped state
+            @param action as Gio.SimpleAction
+            @param state as bool
+        """
+        action.set_state(state)
+        if state:
+            self.__album.set_loved(self.__album.loved | LovedFlags.SKIPPED)
+        else:
+            self.__album.set_loved(self.__album.loved & ~LovedFlags.SKIPPED)
+
+    def __on_play_action_activate(self, action, variant):
         """
             Play album
-            @param Gio.SimpleAction
-            @param GLib.Variant
+            @param action as Gio.SimpleAction
+            @param variant as GLib.Variant
         """
-        App().player.play_album(self.__album)
+        App().player.play_album(self.__album.clone(True))
 
 
 class TrackPlaybackMenu(PlaybackMenu):
@@ -414,6 +642,14 @@ class TrackPlaybackMenu(PlaybackMenu):
             self._set_playback_actions()
             self.__set_queue_actions()
         self.__set_stop_after_action()
+        action = Gio.SimpleAction.new_stateful(
+                "skip-track",
+                None,
+                GLib.Variant.new_boolean(
+                    self.__track.loved & LovedFlags.SKIPPED))
+        App().add_action(action)
+        action.connect("change-state", self.__on_loved_change_state)
+        self.append(_("Ignored"), "app.skip-track")
 
     @property
     def in_player(self):
@@ -436,23 +672,17 @@ class TrackPlaybackMenu(PlaybackMenu):
             @param Gio.SimpleAction
             @param GLib.Variant
         """
-        albums = App().player.albums
-        # If album last in list, merge
-        if albums and albums[-1].id == self.__track.album.id:
-            albums[-1].append_track(self.__track)
-            App().player.set_next()
-        # Add album with only one track
+        album = Album(self.__track.album.id)
+        album.set_tracks([self.__track])
+        App().player.add_album(album)
+        if App().player.is_playing:
+            App().player.add_album(album)
         else:
-            album = Album(self.__track.album.id)
-            album.set_tracks([self.__track])
-            if App().player.is_playing:
-                App().player.add_album(album)
-            else:
-                App().player.play_album(album)
+            App().player.play_album(album)
 
     def _remove_from_playback(self, action, variant):
         """
-            Delete track id from playback
+            Delete track from playback
             @param Gio.SimpleAction
             @param GLib.Variant
         """
@@ -461,10 +691,8 @@ class TrackPlaybackMenu(PlaybackMenu):
                 if self.__track.id in album.track_ids:
                     index = album.track_ids.index(self.__track.id)
                     track = album.tracks[index]
-                    album.remove_track(track)
+                    App().player.remove_track_from_album(track, album)
                     break
-        App().player.set_next()
-        App().player.set_prev()
 
 #######################
 # PRIVATE             #
@@ -530,3 +758,15 @@ class TrackPlaybackMenu(PlaybackMenu):
         """
         App().player.remove_from_queue(self.__track.id, False)
         emit_signal(App().player, "queue-changed")
+
+    def __on_loved_change_state(self, action, state):
+        """
+            Update Skipped state
+            @param action as Gio.SimpleAction
+            @param state as bool
+        """
+        action.set_state(state)
+        if state:
+            self.__track.set_loved(self.__track.loved | LovedFlags.SKIPPED)
+        else:
+            self.__track.set_loved(self.__track.loved & ~LovedFlags.SKIPPED)
