@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -17,7 +17,7 @@ from gettext import gettext as _
 
 from lollypop.tagreader import TagReader, Discoverer
 from lollypop.player_plugins import PluginsPlayer
-from lollypop.define import GstPlayFlags, App
+from lollypop.define import GstPlayFlags, App, StorageType, Repeat
 from lollypop.codecs import Codecs
 from lollypop.logger import Logger
 from lollypop.objects_track import Track
@@ -247,10 +247,7 @@ class BinPlayer:
         try:
             emit_signal(self, "loading-changed", False, self._current_track)
             self._current_track = track
-            # If track_uri is different, preload has happened
-            # See Player.set_next()
-            track_uri = App().tracks.get_uri(track.id)
-            if track.is_web and track.uri == track_uri:
+            if track.is_web and not track.uri_loaded:
                 emit_signal(self, "loading-changed", True, track)
                 self.__load_from_web(track)
                 return False
@@ -283,15 +280,15 @@ class BinPlayer:
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
-        if isinstance(self._current_track, Track):
+        if self._current_track.storage_type != StorageType.EXTERNAL:
             return
         Logger.debug("Player::__on_bus_message_tag(): %s" %
                      self._current_track.uri)
         reader = TagReader()
         tags = message.parse_tag()
         title = reader.get_title(tags, "")
-        if len(title) > 1 and self._current_track.artists != [title]:
-            self._current_track.artists = [title]
+        if len(title) > 1 and self._current_track.title != title:
+            self._current_track.set_name(title)
             emit_signal(self, "current-changed")
 
     def _on_bus_element(self, bus, message):
@@ -347,8 +344,11 @@ class BinPlayer:
             # Don't do anything if crossfade on, track already scrobbled
             # See TransitionsPlayer
             if not self.crossfading:
-                self._on_track_finished(App().player.current_track)
-                if self._next_track.id is not None:
+                self._on_track_finished(self.current_track)
+                repeat = App().settings.get_enum("repeat")
+                if repeat == Repeat.TRACK:
+                    self._load_track(self.current_track)
+                elif self._next_track.id is not None:
                     self._load_track(self._next_track)
         except Exception as e:
             Logger.error("BinPlayer::_on_stream_about_to_finish(): %s", e)
@@ -416,6 +416,7 @@ class BinPlayer:
             return
         if uri:
             track.set_uri(uri)
+            track.set_preloaded()
             self.load(track)
             App().task_helper.run(self.__update_current_duration, track)
         else:
