@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -91,6 +91,7 @@ class MtpSyncDb:
             stream.get_output_stream().write_all(jsondb.encode("utf-8"))
             tmpfile.copy(dbfile, Gio.FileCopyFlags.OVERWRITE, None, None)
             stream.close()
+            tmpfile.delete(None)
         except Exception as e:
             Logger.error("MtpSyncDb::__save(): %s", e)
 
@@ -247,7 +248,7 @@ class MtpSync(GObject.Object):
             album_ids = App().albums.get_synced_ids(0)
             album_ids += App().albums.get_synced_ids(index)
             for album_id in album_ids:
-                album = Album(album_id)
+                album = Album(album_id, skipped=False)
                 tracks += album.tracks
             # New tracks for playlists
             playlist_ids = App().playlists.get_synced_ids(0)
@@ -294,7 +295,7 @@ class MtpSync(GObject.Object):
             emit_signal(self, "sync-progress",
                         self.__done / self.__total + 2)
         except Exception as e:
-            Logger.error("MtpSync::__sync(): %s" % e)
+            Logger.error("MtpSync::sync(): %s" % e)
         finally:
             Logger.info("Save sync db")
             if not self.__cancellable.is_cancelled():
@@ -358,7 +359,7 @@ class MtpSync(GObject.Object):
             (convertion_needed,
              dst_uri) = self.__is_convertion_needed(src_uri, dst_uri)
             uris.append((src_uri, dst_uri))
-            art_uri = App().art.get_album_artwork_uri(track.album)
+            art_uri = App().album_art.get_uri(track.album)
             if art_uri is not None:
                 art_filename = Gio.File.new_for_uri(art_uri).get_basename()
                 art_uris.append((art_uri,
@@ -537,8 +538,7 @@ class MtpSync(GObject.Object):
                             not self.__cancellable.is_cancelled():
                         sleep(1)
                     bus.disconnect_by_func(self.__on_bus_eos)
-                    pipeline.set_state(Gst.State.PAUSED)
-                    pipeline.set_state(Gst.State.READY)
+                    bus.remove_signal_watch()
                     pipeline.set_state(Gst.State.NULL)
                     convert_file.move(
                         dst, Gio.FileCopyFlags.OVERWRITE, None, None)
