@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -17,7 +17,10 @@ from gettext import gettext as _
 
 from lollypop.define import App
 from lollypop.logger import Logger
-from lollypop.utils import format_artist_name
+from lollypop.utils_file import decodeUnicode, splitUnicode
+from lollypop.utils import format_artist_name, get_iso_date_from_string
+from lollypop.tag_frame_text import FrameTextTag
+from lollypop.tag_frame_lang import FrameLangTag
 
 
 class Discoverer:
@@ -29,12 +32,7 @@ class Discoverer:
         """
             Init tag reader
         """
-        self.init_discoverer()
 
-    def init_discoverer(self):
-        """
-            Init discover
-        """
         self._discoverer = GstPbutils.Discoverer.new(10 * Gst.SECOND)
 
     def get_info(self, uri):
@@ -53,6 +51,15 @@ class TagReader:
         Scanner tag reader
     """
 
+    __STRING = ["title", "artist", "composer", "conductor",
+                "musicbrainz-albumid", "musicbrainz-trackid",
+                "musicbrainz-artistid", "musicbrainz-albumartistid",
+                "version", "performer", "artist-sortname",
+                "album-artist-sortname", "interpreted-by", "album-artist",
+                "album", "genre", "lyrics", "publisher"]
+    __INT = ["album-disc-number", "track-number"]
+    __DOUBLE = ["beats-per-minute"]
+
     def __init__(self):
         """
             Init tag reader
@@ -68,9 +75,8 @@ class TagReader:
         """
         if tags is None:
             return GLib.path_get_basename(filepath)
-        (exists, title) = tags.get_string_index("title", 0)
-        # We need to check tag is not just spaces
-        if not exists or not title.strip(" "):
+        title = self.__get(tags, ["title"])
+        if not title:
             title = GLib.path_get_basename(filepath)
         return title
 
@@ -82,13 +88,7 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        artists = []
-        for i in range(tags.get_tag_size("artist")):
-            (exists, read) = tags.get_string_index("artist", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                artists.append(read)
-        return "; ".join(artists)
+        return self.__get(tags, ["artist"])
 
     def get_composers(self, tags):
         """
@@ -98,13 +98,17 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        composers = []
-        for i in range(tags.get_tag_size("composer")):
-            (exists, read) = tags.get_string_index("composer", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                composers.append(read)
-        return "; ".join(composers)
+        return self.__get(tags, ["composer"])
+
+    def get_conductors(self, tags):
+        """
+            Return conductors for tags
+            @param tags as Gst.TagList
+            @return string like "conductor1;conductor2;..."
+        """
+        if tags is None:
+            return _("Unknown")
+        return self.__get(tags, ["conductor"])
 
     def get_mb_id(self, tags, name):
         """
@@ -115,8 +119,7 @@ class TagReader:
         """
         if tags is None or not name:
             return ""
-        (exists, mbid) = tags.get_string_index("musicbrainz-" + name, 0)
-        return mbid or ""
+        return self.__get(tags, ["musicbrainz-" + name])
 
     def get_mb_album_id(self, tags):
         """
@@ -158,8 +161,7 @@ class TagReader:
         """
         if tags is None:
             return ""
-        (exists, version) = tags.get_string_index("version", 0)
-        return version or ""
+        return self.__get(tags, ["version"])
 
     def get_performers(self, tags):
         """
@@ -169,13 +171,7 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        performers = []
-        for i in range(tags.get_tag_size("performer")):
-            (exists, read) = tags.get_string_index("performer", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                performers.append(read)
-        return "; ".join(performers)
+        return self.__get(tags, ["performer"])
 
     def get_artist_sortnames(self, tags):
         """
@@ -185,13 +181,7 @@ class TagReader:
         """
         if tags is None:
             return ""
-        sortnames = []
-        for i in range(tags.get_tag_size("artist-sortname")):
-            (exists, read) = tags.get_string_index("artist-sortname", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                sortnames.append(read)
-        return "; ".join(sortnames)
+        return self.__get(tags, ["artist-sortname"])
 
     def get_album_artist_sortnames(self, tags):
         """
@@ -201,13 +191,7 @@ class TagReader:
         """
         if tags is None:
             return ""
-        sortnames = []
-        for i in range(tags.get_tag_size("album-artist-sortname")):
-            (exists, read) = tags.get_string_index("album-artist-sortname", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                sortnames.append(read)
-        return "; ".join(sortnames)
+        return self.__get(tags, ["album-artist-sortname"])
 
     def get_remixers(self, tags):
         """
@@ -217,19 +201,10 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        remixers = []
-        for i in range(tags.get_tag_size("interpreted-by")):
-            (exists, read) = tags.get_string_index("interpreted-by", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                remixers.append(read)
+        remixers = self.__get(tags, ["interpreted-by"])
         if not remixers:
-            for i in range(tags.get_tag_size("extended-comment")):
-                (exists, read) = tags.get_string_index("extended-comment", i)
-                if exists and read.startswith("REMIXER="):
-                    remixer = read[8:]
-                    remixers.append(remixer)
-        return "; ".join(remixers)
+            remixers = self.__get_extended(tags, ["REMIXER"])
+        return remixers
 
     def get_album_artists(self, tags):
         """
@@ -239,13 +214,7 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        artists = []
-        for i in range(tags.get_tag_size("album-artist")):
-            (exists, read) = tags.get_string_index("album-artist", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                artists.append(read)
-        return "; ".join(artists)
+        return self.__get(tags, ["album-artist"])
 
     def get_album_name(self, tags):
         """
@@ -255,11 +224,10 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        (exists, album_name) = tags.get_string_index("album", 0)
-        # We need to check tag is not just spaces
-        if not exists or not album_name.strip(" "):
-            album_name = _("Unknown")
-        return album_name
+        album = self.__get(tags, ["album"])
+        if not album:
+            album = _("Unknown")
+        return album
 
     def get_genres(self, tags):
         """
@@ -269,15 +237,10 @@ class TagReader:
         """
         if tags is None:
             return _("Unknown")
-        genres = []
-        for i in range(tags.get_tag_size("genre")):
-            (exists, read) = tags.get_string_index("genre", i)
-            # We need to check tag is not just spaces
-            if exists and read.strip(" "):
-                genres.append(read)
+        genres = self.__get(tags, ["genre"])
         if not genres:
-            return _("Unknown")
-        return "; ".join(genres)
+            genres = _("Unknown")
+        return genres
 
     def get_discname(self, tags):
         """
@@ -285,18 +248,7 @@ class TagReader:
             @param tags as Gst.TagList
             @return disc name as str
         """
-        if tags is None:
-            return 0
-        discname = ""
-        for i in range(tags.get_tag_size("extended-comment")):
-            (exists, read) = tags.get_string_index("extended-comment", i)
-            if exists and read.startswith("PART"):
-                discname = "=".join(read.split("=")[1:])
-                break
-            if exists and read.startswith("DISCSUBTITLE"):
-                discname = "=".join(read.split("=")[1:])
-                break
-        return discname
+        return self.__get_extended(tags, ['PART', 'DISCSUBTITLE'])
 
     def get_discnumber(self, tags):
         """
@@ -306,11 +258,29 @@ class TagReader:
         """
         if tags is None:
             return 0
-        (exists, discnumber) = tags.get_uint_index("album-disc-number", 0)
-        if not exists:
+        discnumber = self.__get(tags, ["album-disc-number"])
+        if not discnumber:
             discnumber = 0
         return discnumber
 
+    def get_compilation(self, tags):
+        """
+            Return True if album is a compilation
+            @param tags as Gst.TagList
+            @return bool
+        """
+        if tags is None:
+            return False
+        try:
+            compilation = self.__get_private_string(tags, "TCMP", False)
+            if not compilation:
+                compilation = self.__get_extended(tags, ["COMPILATION"])
+            if compilation:
+                return bool(compilation)
+        except Exception as e:
+            Logger.error("TagReader::get_compilation(): %s" % e)
+        return False
+
     def get_tracknumber(self, tags, filename):
         """
             Return track number for tags
@@ -319,10 +289,10 @@ class TagReader:
             @return track number as int
         """
         if tags is not None:
-            (exists, tracknumber) = tags.get_uint_index("track-number", 0)
+            tracknumber = self.__get(tags, ["track-number"])
         else:
-            (exists, tracknumber) = (False, 0)
-        if not exists:
+            tracknumber = None
+        if not tracknumber:
             # Guess from filename
             m = match("^([0-9]*)[ ]*-", filename)
             if m:
@@ -378,47 +348,29 @@ class TagReader:
             @return year and timestamp (int, int)
         """
         def get_id3():
+            date_string = self.__get_private_string(tags, "TDOR", False)
             try:
-                size = tags.get_tag_size("private-id3v2-frame")
-                for i in range(0, size):
-                    (exists, sample) = tags.get_sample_index(
-                        "private-id3v2-frame",
-                        i)
-                    if not exists:
-                        continue
-                    (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
-                    if not exists:
-                        continue
-                    string = m.data.decode("utf-8")
-                    if string.startswith("TDOR"):
-                        split = string.split("\x00")
-                        date = split[-1]
-                        datetime = GLib.DateTime.new_from_iso8601(date, None)
-                        return (datetime.year(), datetime.to_unix())
+                date = get_iso_date_from_string(date_string)
+                datetime = GLib.DateTime.new_from_iso8601(date, None)
+                return (datetime.get_year(), datetime.to_unix())
             except:
                 pass
             return (None, None)
 
         def get_ogg():
             try:
-                size = tags.get_tag_size("extended-comment")
-                for i in range(0, size):
-                    (exists, sample) = tags.get_string_index(
-                        "extended-comment",
-                        i)
-                    if not exists or not sample.startswith("ORIGINALDATE="):
-                        continue
-                    date = sample[13:]
-                    datetime = GLib.DateTime.new_from_iso8601(date, None)
-                    return (datetime.year(), datetime.to_unix())
+                date_string = self.__get_extended(tags, ['ORIGINALDATE'])
+                date = get_iso_date_from_string(date_string)
+                datetime = GLib.DateTime.new_from_iso8601(date, None)
+                return (datetime.get_year(), datetime.to_unix())
             except:
                 pass
-            return None
+            return (None, None)
 
         if tags is None:
             return None
         values = get_id3()
-        if values is None:
+        if values[0] is None:
             values = get_ogg()
         return values
 
@@ -428,89 +380,57 @@ class TagReader:
             @param tags as Gst.TagList
             @return int/None
         """
-        try:
-            if tags is not None:
-                (exists, bpm) = tags.get_double_index("beats-per-minute", 0)
-                if exists:
-                    return bpm
-        except:
-            pass
-        return None
+        bpm = self.__get(tags, ["beats-per-minute"])
+        if not bpm:
+            bpm = None
+        return bpm
 
     def get_popm(self, tags):
         """
             Get popularity tag
             @param tags as Gst.TagList
-            @return value as Lollypop rating
+            @return int
         """
-        if tags is None:
-            return 0
-        size = tags.get_tag_size("private-id3v2-frame")
-        for i in range(0, size):
-            (exists, sample) = tags.get_sample_index("private-id3v2-frame", i)
-            if not exists:
-                continue
-            (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
-            if not exists:
-                continue
-            if m.data[0:4] == b"POPM":
-                # Get tag
-                """
-                    Location of rating for common media players:
-                    - Plain POPM is located in m.data[11]
-                    - Media Monkey is located in m.data[19] and
-                      we are searching m.data[10:40] for "no@email"
-                    - WinAMP is located in m.data[28] and we are searching
-                      m.data[10:40] for "rating@winamp.com"
-                    - Windows Media Player is located in m.data[40] and
-                      we are searching m.data[10:40] for
-                      "Windows Media Player 9 Series"
-                    - MusicBee is located in m.data[19] and
-                      we are searching m.data[10:40] for
-                      "MusicBee"
-                """
-                if m.data[10:40].find(b"Windows Media Player 9 Series") >= 0:
-                    location = 40
-                    Logger.debug(
-                        "Rating type: Windows Media Player Location:"
-                        " %s Rating: %s", location, m.data[location])
-                elif m.data[10:40].find(b"rating@winamp.com") >= 0:
-                    location = 28
-                    Logger.debug(
-                        "Rating type: WinAMP Location: %s Rating: %s",
-                        location, m.data[location])
-                elif m.data[10:40].find(b"no@email") >= 0:
-                    location = 19
-                    Logger.debug(
-                        "Rating type: MediaMonkey Location: %s Rating: %s",
-                        location, m.data[location])
-                elif m.data[10:40].find(b"MusicBee") >= 0:
-                    location = 19
-                    Logger.debug(
-                        "Rating type: MusicBee Location: %s Rating: %s",
-                        location, m.data[location])
-                else:
-                    location = 11
-                    Logger.debug(
-                        "Rating type: none Location: %s Rating: %s",
-                        location, m.data[location])
+        try:
+            if tags is None:
+                return 0
+            size = tags.get_tag_size("private-id3v2-frame")
+            for i in range(0, size):
+                (exists, sample) = tags.get_sample_index("private-id3v2-frame",
+                                                         i)
+                if not exists:
+                    continue
+                (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
+                if not exists:
+                    continue
+                # Gstreamer 1.18 API breakage
+                try:
+                    bytes = m.data.tobytes()
+                except:
+                    bytes = m.data
 
-                popm = m.data[location]
-                if popm == 0:
-                    value = 0
-                elif popm == 1:
-                    value = 1
-                elif popm == 64:
-                    value = 2
-                elif popm == 128:
-                    value = 3
-                elif popm == 196:
-                    value = 4
-                elif popm == 255:
-                    value = 5
-                else:
-                    value = 0
-                return value
+                if len(bytes) > 4 and bytes[0:4] == b"POPM":
+                    try:
+                        popm = bytes.split(b"\x00")[6][0]
+                    except:
+                        popm = 0
+                    if popm == 0:
+                        value = 0
+                    elif popm >= 1 and popm < 64:
+                        value = 1
+                    elif popm >= 64 and popm < 128:
+                        value = 2
+                    elif popm >= 128 and popm < 196:
+                        value = 3
+                    elif popm >= 196 and popm < 255:
+                        value = 4
+                    elif popm == 255:
+                        value = 5
+                    else:
+                        value = 0
+                    return value
+        except Exception as e:
+            Logger.warning("TagReader::get_popm(): %s", e)
         return 0
 
     def get_lyrics(self, tags):
@@ -519,60 +439,14 @@ class TagReader:
             @parma tags as Gst.TagList
             @return lyrics as str
         """
-        def decode_lyrics(bytes):
-            from lollypop.utils_file import decodeUnicode, splitUnicode
-            try:
-                prefix = bytes[0:4]
-                if prefix in [b"USLT"]:
-                    frame = bytes[10:]
-                    encoding = frame[0:1]
-                    (d, t) = splitUnicode(frame[4:], encoding)
-                    return decodeUnicode(t, encoding)
-            except Exception as e:
-                Logger.warning("TagReader::get_lyrics(): %s", e)
-            return None
-
         def get_mp4():
-            try:
-                (exists, sample) = tags.get_string_index("lyrics", 0)
-                if exists:
-                    return sample
-            except Exception as e:
-                Logger.error("TagReader::get_mp4(): %s" % e)
-            return ""
+            return self.__get(tags, ["lyrics"])
 
         def get_id3():
-            try:
-                size = tags.get_tag_size("private-id3v2-frame")
-                for i in range(0, size):
-                    (exists, sample) = tags.get_sample_index(
-                        "private-id3v2-frame",
-                        i)
-                    if not exists:
-                        continue
-                    (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
-                    if not exists:
-                        continue
-                    string = decode_lyrics(m.data)
-                    if string is not None:
-                        return string
-            except Exception as e:
-                Logger.error("TagReader::get_id3(): %s" % e)
-            return ""
+            return self.__get_private_string(tags, "USLT", True)
 
         def get_ogg():
-            try:
-                size = tags.get_tag_size("extended-comment")
-                for i in range(0, size):
-                    (exists, sample) = tags.get_string_index(
-                        "extended-comment",
-                        i)
-                    if not exists or not sample.startswith("LYRICS="):
-                        continue
-                    return sample[7:]
-            except Exception as e:
-                Logger.error("TagReader::get_ogg(): %s" % e)
-            return ""
+            return self.__get_extended(tags, ["LYRICS"])
 
         if tags is None:
             return ""
@@ -591,7 +465,6 @@ class TagReader:
         """
         def decode_lyrics(bytes_list, encoding):
             lyrics = []
-            from lollypop.utils_file import decodeUnicode, splitUnicode
             try:
                 for frame in bytes_list:
                     (l, t) = splitUnicode(frame, encoding)
@@ -599,31 +472,21 @@ class TagReader:
                         lyrics.append((decodeUnicode(l, encoding),
                                        int.from_bytes(t[1:4], "big")))
             except Exception as e:
-                Logger.warning("TagReader::get_synced_lyrics1(): %s", e)
+                Logger.warning(
+                        "TagReader::get_synced_lyrics.decode_lyrics(): %s", e)
             return lyrics
 
         def get_id3():
             try:
-                size = tags.get_tag_size("private-id3v2-frame")
-                for i in range(0, size):
-                    (exists, sample) = tags.get_sample_index(
-                        "private-id3v2-frame",
-                        i)
-                    if not exists:
-                        continue
-                    (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
-                    if not exists:
-                        continue
-                    prefix = (m.data[0:4])
-                    if prefix not in [b"SYLT"]:
-                        continue
-                    frame = m.data[10:]
+                b = self.__get_private_bytes(tags, "SYLT")
+                if b:
+                    frame = b[10:]
                     encoding = frame[0:1]
                     string = decode_lyrics(frame.split(b"\n"), encoding)
                     if string is not None:
                         return string
             except Exception as e:
-                Logger.error("TagReader::get_synced_lyrics2(): %s" % e)
+                Logger.warning("TagReader::get_synced_lyrics.get_id3(): %s", e)
             return ""
 
         if tags is None:
@@ -631,12 +494,12 @@ class TagReader:
         lyrics = get_id3()
         return lyrics
 
-    def add_artists(self, artists, sortnames, mb_artist_ids=""):
+    def add_artists(self, artists, sortnames, mb_artist_id=""):
         """
             Add artists to db
-            @param artists as [string]
-            @param sortnames as [string]
-            @param mb_artist_ids as [string]
+            @param artists as str
+            @param sortnames as str
+            @param mb_artist_id as str
             @return ([int], [int]): (added artist ids, artist ids)
         """
         artist_ids = []
@@ -644,7 +507,7 @@ class TagReader:
         artistsplit = artists.split(";")
         sortsplit = sortnames.split(";")
         sortlen = len(sortsplit)
-        mbidsplit = mb_artist_ids.split(";")
+        mbidsplit = mb_artist_id.split(";")
         mbidlen = len(mbidsplit)
         if len(artistsplit) != mbidlen:
             mbidsplit = []
@@ -700,12 +563,13 @@ class TagReader:
                 genre_ids.append(genre_id)
         return (added_genre_ids, genre_ids)
 
-    def add_album(self, album_name, mb_album_id, artist_ids,
+    def add_album(self, album_name, mb_album_id, lp_album_id, artist_ids,
                   uri, loved, popularity, rate, synced, mtime, storage_type):
         """
             Add album to db
             @param album_name as str
             @param mb_album_id as str
+            @param lp_album_id as str
             @param artist_ids as [int]
             @param uri as str
             @param loved as bool
@@ -718,16 +582,134 @@ class TagReader:
             @commit needed
         """
         added = False
-        f = Gio.File.new_for_uri(uri)
-        d = f.get_parent()
-        parent_uri = "" if d is None else d.get_uri()
+        if uri.find("://") != -1:
+            f = Gio.File.new_for_uri(uri)
+            parent = f.get_parent()
+            if parent is not None:
+                uri = parent.get_uri()
         album_id = App().albums.get_id(album_name, mb_album_id, artist_ids)
+        # Check storage type did not changed, remove album then
+        if album_id is not None:
+            current_storage_type = App().albums.get_storage_type(album_id)
+            if current_storage_type != storage_type:
+                App().tracks.remove_album(album_id)
+                App().tracks.clean(False)
+                App().albums.clean(False)
+                App().artists.clean(False)
+                album_id = None
         if album_id is None:
             added = True
-            album_id = App().albums.add(album_name, mb_album_id, artist_ids,
-                                        parent_uri, loved, popularity,
+            album_id = App().albums.add(album_name, mb_album_id, lp_album_id,
+                                        artist_ids, uri, loved, popularity,
                                         rate, synced, mtime, storage_type)
-        # Now we have our album id, check if path doesn"t change
-        if App().albums.get_uri(album_id) != parent_uri:
-            App().albums.set_uri(album_id, parent_uri)
+        # Check if path did not change
+        elif App().albums.get_uri(album_id) != uri:
+            App().albums.set_uri(album_id, uri)
         return (added, album_id)
+
+#######################
+# PRIVATE             #
+#######################
+    def __get_extended(self, tags, keys):
+        """
+            Return tag from tags following keys
+            @param tags as Gst.TagList
+            @param keys as [str]
+            @return Tag as str
+        """
+        if tags is None:
+            return ""
+        items = []
+        try:
+            for i in range(tags.get_tag_size("extended-comment")):
+                (exists, read) = tags.get_string_index("extended-comment", i)
+                for key in keys:
+                    if exists and read.startswith(key + "="):
+                        items.append("".join(read.split("=")[1:]))
+        except Exception as e:
+            Logger.error("TagReader::__get_extended(): %s", e)
+        return ";".join(items)
+
+    def __get(self, tags, keys):
+        """
+            Return tag from tags following keys
+            Only handles string/uint/double
+            @param tags as Gst.TagList
+            @param keys as [str]
+            @return Tag as str/int/double. Empty string if does not exist
+        """
+        if tags is None:
+            return ""
+        items = []
+        try:
+            for key in keys:
+                for i in range(tags.get_tag_size(key)):
+                    if key in self.__STRING:
+                        (exists, read) = tags.get_string_index(key, i)
+                        if exists and read.strip(" "):
+                            items.append(read)
+                    elif key in self.__INT:
+                        (exists, read) = tags.get_uint_index(key, i)
+                        if exists:
+                            return read
+                    elif key in self.__DOUBLE:
+                        (exists, read) = tags.get_double_index(key, i)
+                        if exists:
+                            return read
+                    else:
+                        Logger.error("Missing key" % key)
+        except Exception as e:
+            Logger.error("TagReader::__get(): %s", e)
+        return ";".join(items)
+
+    def __get_private_bytes(self, tags, key):
+        """
+            Get key from private frame
+            @param tags as Gst.TagList
+            @param key as str
+            @return frame as bytes
+        """
+        try:
+            size = tags.get_tag_size("private-id3v2-frame")
+            encoded_key = key.encode("utf-8")
+            for i in range(0, size):
+                (exists, sample) = tags.get_sample_index(
+                    "private-id3v2-frame",
+                    i)
+                if not exists:
+                    continue
+                (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
+                if not exists:
+                    continue
+                # Gstreamer 1.18 API breakage
+                try:
+                    b = m.data.tobytes()
+                except:
+                    b = m.data
+
+                if b[0:len(encoded_key)] != encoded_key:
+                    continue
+                return b
+        except Exception as e:
+            Logger.error("TagReader::__get_private_bytes(): %s" % e)
+        return b""
+
+    def __get_private_string(self, tags, key, lang):
+        """
+            Get key from private frame
+            @param tags as Gst.TagList
+            @param key as str
+            @param lang as bool
+            @return Tag as str
+        """
+        try:
+            b = self.__get_private_bytes(tags, key)
+            if lang:
+                frame = FrameLangTag(b)
+            else:
+                frame = FrameTextTag(b)
+            if frame.key == key:
+                return frame.string
+        except Exception as e:
+            Logger.error("TagReader::__get_private(): %s" % e)
+        return ""
