@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -13,6 +13,7 @@
 from gi.repository import Gio
 
 from gettext import gettext as _
+from time import time
 from random import shuffle
 
 from lollypop.view_flowbox import FlowBoxView
@@ -20,14 +21,14 @@ from lollypop.widgets_album_simple import AlbumSimpleWidget
 from lollypop.define import App, Type, ViewType, ScanUpdate, StorageType
 from lollypop.objects_album import Album
 from lollypop.utils import get_icon_name, get_network_available, popup_widget
-from lollypop.utils import get_font_height, get_title_for_genres_artists
+from lollypop.utils import get_title_for_genres_artists
+from lollypop.utils import remove_static
 from lollypop.utils_file import get_youtube_dl
 from lollypop.utils_album import get_album_ids_for
-from lollypop.controller_view import ViewController, ViewControllerType
 from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
+class AlbumsBoxView(FlowBoxView, SignalsHelper):
     """
         Show albums in a box
     """
@@ -42,7 +43,7 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
             @param view_type as ViewType
         """
         FlowBoxView.__init__(self, storage_type, view_type)
-        ViewController.__init__(self, ViewControllerType.ALBUM)
+        self.__time = time() + 10
         self._genre_ids = genre_ids
         self._artist_ids = artist_ids
         self._storage_type = storage_type
@@ -50,24 +51,21 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
         if genre_ids and genre_ids[0] < 0:
             if genre_ids[0] == Type.WEB:
                 (youtube_dl, env) = get_youtube_dl()
-                if not Gio.NetworkMonitor.get_default(
-                        ).get_network_available():
-                    self._empty_message = _("Network not available")
-                    self.show_placeholder(True)
-                    self.__populate_wanted = False
-                elif youtube_dl is None:
+                if youtube_dl is None:
                     self._empty_message = _("Missing youtube-dl command")
                     self.show_placeholder(True)
                     self.__populate_wanted = False
                 elif not get_network_available("YOUTUBE"):
                     self._empty_message =\
-                        _("You need to enable YouTube in network settings")
+                        _("Network unavailable or disabled in settings")
                     self.show_placeholder(True)
                     self.__populate_wanted = False
             self._empty_icon_name = get_icon_name(genre_ids[0])
         return [
-            (App().scanner, "album-updated", "_on_album_updated"),
-            (App().player, "loading-changed", "_on_loading_changed")
+            (App().scanner, "updated", "_on_collection_updated"),
+            (App().player, "loading-changed", "_on_loading_changed"),
+            (App().player, "current-changed", "_on_current_changed"),
+            (App().album_art, "album-artwork-changed", "_on_artwork_changed")
         ]
 
     def populate(self, albums=[]):
@@ -84,11 +82,20 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
                 self.show_placeholder(True)
 
         def load():
+            # No skipped albums for this views
+            if self._genre_ids and self._genre_ids[0] in [Type.POPULARS,
+                                                          Type.LITTLE,
+                                                          Type.RANDOMS,
+                                                          Type.RECENTS]:
+                skipped = False
+            else:
+                skipped = True
             album_ids = get_album_ids_for(self._genre_ids, self._artist_ids,
-                                          self.storage_type)
+                                          self.storage_type, skipped)
             albums = []
             for album_id in album_ids:
-                album = Album(album_id, self._genre_ids, self._artist_ids)
+                album = Album(album_id, self._genre_ids,
+                              self._artist_ids, True)
                 album.set_storage_type(self.storage_type)
                 albums.append(album)
             return albums
@@ -98,19 +105,13 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
         elif self.__populate_wanted:
             App().task_helper.run(load, callback=(on_load,))
 
-    def insert_album(self, album, position):
+    def add_value(self, album):
         """
             Add a new album
             @param album as Album
-            @param position as int
-            @param cover_uri as int
         """
-        widget = AlbumSimpleWidget(album, self._genre_ids,
-                                   self._artist_ids, self.view_type,
-                                   get_font_height())
-        self._box.insert(widget, position)
-        widget.show()
-        widget.populate()
+        self.show_placeholder(False)
+        FlowBoxView.add_value_unsorted(self, album)
 
     def clear(self):
         """
@@ -128,23 +129,23 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
         return {"genre_ids": self._genre_ids,
                 "artist_ids": self._artist_ids,
                 "storage_type": self._storage_type,
-                "view_type": self.view_type & ~(ViewType.ADAPTIVE |
-                                                ViewType.SMALL)}
+                "view_type": self.view_type & ~ViewType.SMALL}
 
 #######################
 # PROTECTED           #
 #######################
-    def _get_child(self, value):
+    def _get_child(self, value, position=-1):
         """
             Get a child for view
             @param value as object
+            @param position as int
             @return row as SelectionListRow
         """
         if self.destroyed:
             return None
         widget = AlbumSimpleWidget(value,  self._genre_ids, self._artist_ids,
                                    self.view_type, self.font_height)
-        self._box.insert(widget, -1)
+        self._box.insert(widget, position)
         widget.show()
         return widget
 
@@ -156,31 +157,45 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
         """
         from lollypop.widgets_menu import MenuBuilder
         from lollypop.menu_objects import AlbumMenu
-        menu = AlbumMenu(child.data, self.storage_type,
-                         self.view_type, App().window.is_adaptive)
+        menu = AlbumMenu(child.data, self.storage_type, self.view_type)
         return MenuBuilder(menu)
 
-    def _on_album_updated(self, scanner, album_id, scan_update):
+    def _on_current_changed(self, player):
+        """
+            Update children state
+            @param player as Player
+        """
+        for child in self._box.get_children():
+            child.set_selection()
+
+    def _on_collection_updated(self, scanner, item, scan_update):
         """
             Handles changes in collection
             @param scanner as CollectionScanner
-            @param album_id as int
+            @param item as CollectionItem
             @param scan_update as ScanUpdate
         """
-        if scan_update == ScanUpdate.ADDED:
-            album_ids = get_album_ids_for(self._genre_ids, self._artist_ids,
-                                          self.storage_type)
-            if album_id in album_ids:
-                index = album_ids.index(album_id)
-                self.insert_album(Album(album_id), index)
-        elif scan_update == ScanUpdate.MODIFIED:
-            for child in self.children:
-                if child.data.id == album_id:
-                    child.data.reset_tracks()
-                    break
+        # On first update, ignore notifications for 10 seconds
+        # Next, ignore notifications for 120 seconds
+        if scan_update == ScanUpdate.ADDED and time() > self.__time:
+            wanted = True
+            for genre_id in item.genre_ids:
+                genre_ids = remove_static(self._genre_ids)
+                if genre_ids and genre_id not in genre_ids:
+                    wanted = False
+            for artist_id in item.artist_ids:
+                artist_ids = remove_static(self._artist_ids)
+                if artist_ids and artist_id not in artist_ids:
+                    wanted = False
+            if wanted:
+                self.__time = time() + 120
+                App().window.container.show_notification(
+                    _("New albums available"),
+                    [_("Refresh")],
+                    [App().window.container.reload_view])
         elif scan_update == ScanUpdate.REMOVED:
             for child in self.children:
-                if child.data.id == album_id:
+                if child.data.id == item.album_id:
                     child.destroy()
                     break
 
@@ -202,6 +217,8 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
             @param track as Track
         """
         for child in self.children:
+            if child.artwork is None:
+                continue
             if child.data.id == track.album.id:
                 context = child.artwork.get_style_context()
                 if status:
@@ -246,11 +263,10 @@ class AlbumsBoxView(FlowBoxView, ViewController, SignalsHelper):
 
         def play_album(status, child):
             child.artwork.get_style_context().remove_class("load-animation")
-            child.data.reset_tracks()
-            App().player.play_album(child.data.get_with_skipping_allowed())
+            App().player.play_album(child.data.clone(False))
 
         if child.data.storage_type & StorageType.COLLECTION:
-            App().player.play_album(child.data.get_with_skipping_allowed())
+            App().player.play_album(child.data.clone(False))
         else:
             child.artwork.get_style_context().add_class("load-animation")
             cancellable = Gio.Cancellable.new()
@@ -303,7 +319,7 @@ class AlbumsForGenresBoxView(AlbumsBoxView):
             @param banner as AlbumsBannerWidget
             @param random as bool
         """
-        albums = [c.data for c in self._box.get_children()]
+        albums = [c.data.clone(False) for c in self._box.get_children()]
         if not albums:
             return
         if random:
@@ -325,10 +341,10 @@ class AlbumsForGenresBoxView(AlbumsBoxView):
             if child.data.storage_type & StorageType.COLLECTION:
                 albums.append(child.data)
         title = get_title_for_genres_artists(self._genre_ids, self._artist_ids)
-        menu = AlbumsMenu(title, albums, App().window.is_adaptive)
+        menu = AlbumsMenu(title, albums, self.view_type)
         menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popup_widget(menu_widget, button)
+        popup_widget(menu_widget, button, None, None, button)
 
 
 class AlbumsForYearsBoxView(AlbumsForGenresBoxView):
@@ -354,14 +370,22 @@ class AlbumsForYearsBoxView(AlbumsForGenresBoxView):
         def on_load(items):
             FlowBoxView.populate(self, items)
 
+        def get_album(album_id, disc_number, disc_name, album_year, year):
+            album = Album(album_id, [Type.YEARS], [])
+            if year != album_year:
+                album.set_disc_number(disc_number)
+            return album
+
         def load():
-            items = []
+            albums = []
             for year in self._artist_ids:
-                items += App().albums.get_compilations_for_year(
-                    year, self.storage_type)
-                items += App().albums.get_albums_for_year(
-                    year, self.storage_type)
-            return [Album(album_id, [Type.YEARS], []) for album_id in items]
+                items = App().tracks.get_compilations_by_disc_for_year(
+                    year, self.storage_type, True)
+                items += App().tracks.get_albums_by_disc_for_year(
+                    year, self.storage_type, True)
+                albums += [get_album(item[0], item[1], item[2], item[3], year)
+                           for item in items]
+            return albums
 
         App().task_helper.run(load, callback=(on_load,))
 
