@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -46,7 +46,7 @@ class DecadesBoxView(FlowBoxView):
             FlowBoxView.populate(self, items)
 
         def load():
-            (years, unknown) = App().albums.get_years(self.storage_type)
+            (years, unknown) = App().tracks.get_years(self.storage_type)
             decades = []
             decade = []
             current_d = None
