@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -21,7 +21,6 @@ from lollypop.objects_album import Album
 from lollypop.information_store import InformationStore
 from lollypop.view_albums_list import AlbumsListView
 from lollypop.view import View
-from lollypop.utils import get_network_available
 from lollypop.utils import get_default_storage_type
 from lollypop.widgets_banner_information import InformationBannerWidget
 from lollypop.helper_signals import SignalsHelper, signals_map
@@ -104,7 +103,8 @@ class InformationView(View, SignalsHelper):
             self.__banner.show()
             self.__banner.connect("search", self.__on_banner_search)
             self.__albums_view = AlbumsListView([], [],
-                                                ViewType.SCROLLED)
+                                                ViewType.SCROLLED |
+                                                ViewType.ARTIST)
             self.__albums_view.set_size_request(350, -1)
             self.__albums_view.show()
             self.__albums_view.set_halign(Gtk.Align.END)
@@ -230,19 +230,15 @@ class InformationView(View, SignalsHelper):
             @param status as bool
         """
         if status:
-            self.__label.set_text(_("Loading information"))
-            if get_network_available("WIKIPEDIA"):
-                from lollypop.helper_web_wikipedia import WikipediaHelper
-                wikipedia = WikipediaHelper()
-                self.__label.hide()
-                self.__listbox.show()
-                self.__stack.set_visible_child(self.__listbox)
-                App().task_helper.run(
-                    wikipedia.get_search_list,
-                    self.__artist_name,
-                    callback=(self.__on_wikipedia_search_list,))
-            else:
-                self.__banner.button.toggled()
+            from lollypop.helper_web_wikipedia import WikipediaHelper
+            wikipedia = WikipediaHelper()
+            self.__label.hide()
+            self.__listbox.show()
+            self.__stack.set_visible_child(self.__listbox)
+            App().task_helper.run(
+                wikipedia.get_search_list,
+                self.__artist_name,
+                callback=(self.__on_wikipedia_search_list,))
         else:
             self.__show_main_widget()
 
@@ -281,7 +277,6 @@ class InformationView(View, SignalsHelper):
             self.__artist_artwork.hide()
         else:
             self.__artist_artwork.set_from_surface(surface)
-            del surface
 
     def __on_artist_information(self, content, artist_name):
         """
