@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -27,15 +27,17 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
     """
     _ALBUMS_COUNT = 10
 
-    def __init__(self, data, name, sortname, view_type):
+    def __init__(self, data, name, sortname, view_type, font_height):
         """
             Init widget
             @param data as object
             @param name as str
             @param sortname as str
-            @param art_size as int
+            @param view_type as ViewType
+            @param font_height as int
         """
-        RoundedFlowBoxWidget.__init__(self, data, name, sortname, view_type)
+        RoundedFlowBoxWidget.__init__(self, data, name, sortname,
+                                      view_type, font_height)
         self._genre = Type.NONE
         self.__album_ids = []
         self.__cancellable = Gio.Cancellable()
@@ -49,25 +51,17 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         RoundedFlowBoxWidget.populate(self)
         self._artwork.get_style_context().add_class("rounded-icon-large")
 
-    def set_view_type(self, view_type):
-        """
-            Update artwork size
-            @param view_type as ViewType
-        """
-        RoundedFlowBoxWidget.set_view_type(self, view_type)
-        self._pixel_size = self._art_size / 8
-
     def set_artwork(self):
         """
             Set artwork
         """
         RoundedFlowBoxWidget.set_artwork(self)
-        if App().art.artwork_exists_in_cache(self.artwork_name,
-                                             "ROUNDED",
-                                             self._art_size,
-                                             self._art_size):
+        if App().art.exists_in_cache(self.artwork_name,
+                                     "ROUNDED",
+                                     self._art_size,
+                                     self._art_size):
             App().task_helper.run(
-                App().art.get_artwork_from_cache,
+                App().art.get_from_cache,
                 self.artwork_name, "ROUNDED",
                 self._art_size, self._art_size,
                 callback=(self.__on_load_from_cache,))
@@ -95,11 +89,12 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         album_ids = list(self.__album_ids)
         album_pixbufs = []
         album_scaled_pixbufs = []
-        for album in album_ids:
-            pixbuf = App().art.get_album_artwork(Album(album),
-                                                 self._art_size,
-                                                 self._art_size,
-                                                 self._scale_factor)
+        while album_ids and len(album_pixbufs) != 9:
+            album_id = album_ids.pop(0)
+            pixbuf = App().album_art.get(Album(album_id),
+                                         self._art_size,
+                                         self._art_size,
+                                         self._scale_factor)
             if pixbuf is not None:
                 album_pixbufs.append(pixbuf)
         if len(album_pixbufs) == 0:
@@ -122,7 +117,6 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             newpix = pixbuf.scale_simple(self.__cover_size,
                                          self.__cover_size,
                                          GdkPixbuf.InterpType.NEAREST)
-            del pixbuf
             album_scaled_pixbufs.append(newpix)
 
         if len(album_scaled_pixbufs) == 0:
@@ -162,12 +156,11 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             return
         rounded = get_round_surface(
             surface, self._scale_factor, self._art_size / 4)
-        del surface
         self._artwork.set_from_surface(rounded)
-        App().art.add_artwork_to_cache(self.artwork_name,
-                                       rounded,
-                                       "ROUNDED")
-        del rounded
+        App().art.add_to_cache(self.artwork_name,
+                               rounded,
+                               "ROUNDED",
+                               self._artwork.get_scale_factor())
         emit_signal(self, "populated")
 
     def __draw_surface(self, surface, ctx, positions,
@@ -190,14 +183,12 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             y *= self.__cover_size
             subsurface = Gdk.cairo_surface_create_from_pixbuf(
                 pixbuf, self._scale_factor, None)
-            del pixbuf
             ctx.translate(x, y)
             ctx.set_source_surface(subsurface, 0, 0)
             ctx.paint()
             ctx.translate(-x, -y)
             self.__draw_surface(surface, ctx, positions,
                                 album_pixbufs, set_surface)
-            del surface
         if self.__cancellable.is_cancelled():
             return
         elif album_pixbufs and len(positions) > 0:
