@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,13 +16,11 @@ from gettext import gettext as _
 
 from lollypop.view_tracks_album import AlbumTracksView
 from lollypop.define import ArtSize, App, ViewType, MARGIN_SMALL
-from lollypop.define import ArtBehaviour, StorageType
-from lollypop.utils import popup_widget, emit_signal
-from lollypop.helper_gestures import GesturesHelper
-from lollypop.helper_signals import SignalsHelper, signals_map
+from lollypop.define import ArtBehaviour
+from lollypop.utils import emit_signal
 
 
-class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
+class AlbumRow(Gtk.ListBoxRow):
     """
         Album row
     """
@@ -31,6 +29,8 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         "activated": (GObject.SignalFlags.RUN_FIRST,
                       None, (GObject.TYPE_PYOBJECT,)),
         "populated": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "track-removed": (GObject.SignalFlags.RUN_FIRST, None,
+                          (GObject.TYPE_PYOBJECT,)),
     }
 
     def get_best_height(widget):
@@ -51,7 +51,6 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         else:
             return cover_height + 4
 
-    @signals_map
     def __init__(self, album, height, view_type):
         """
             Init row widgets
@@ -73,19 +72,7 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         context_style.add_class("albumrow-collapsed")
         self.set_property("height-request", height)
         self.connect("destroy", self.__on_destroy)
-        self.__tracks_view = AlbumTracksView(self.__album,
-                                             self.__view_type |
-                                             ViewType.SINGLE_COLUMN)
-        self.__tracks_view.connect("activated",
-                                   self.__on_tracks_view_activated)
-        self.__tracks_view.connect("populated",
-                                   self.__on_tracks_view_populated)
-        self.__tracks_view.connect("track-removed",
-                                   self.__on_tracks_view_track_removed)
-        self.__tracks_view.show()
-        return [
-            (App().player, "playback-changed", "_on_playback_changed")
-        ]
+        self.__tracks_view = self.__get_new_tracks_view()
 
     def populate(self):
         """
@@ -98,7 +85,10 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         App().art_helper.set_frame(self.__artwork, "small-cover-frame",
                                    ArtSize.SMALL, ArtSize.SMALL)
         self.set_sensitive(True)
-        self.set_property("has-tooltip", True)
+        # Issue with gtk3-3.24.24, tooltip is flashing when in
+        # information popover
+        self.set_property("has-tooltip",
+                          self.get_ancestor(Gtk.Popover) is None)
         self.connect("query-tooltip", self.__on_query_tooltip)
         if self.__album.artists:
             artists = GLib.markup_escape_text(", ".join(self.__album.artists))
@@ -113,54 +103,35 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         self.__title_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.__title_label.set_property("halign", Gtk.Align.START)
         self.__title_label.get_style_context().add_class("dim-label")
-        buttons = []
-        if self.__view_type & ViewType.SEARCH:
+        if self.__view_type & (ViewType.PLAYBACK | ViewType.PLAYLISTS):
+            button = Gtk.Button.new_from_icon_name("list-remove-symbolic",
+                                                   Gtk.IconSize.BUTTON)
+            if self.__view_type & ViewType.PLAYBACK:
+                button.set_tooltip_text(_("Remove from playback"))
+            else:
+                button.set_tooltip_text(_("Remove from playlist"))
+            button.connect("clicked", self.__on_remove_clicked)
+        else:
             button = Gtk.Button.new_from_icon_name(
                     "media-playback-start-symbolic",
                     Gtk.IconSize.MENU)
             button.set_tooltip_text(_("Play this album"))
-            self.__gesture_play = GesturesHelper(
-                button,
-                primary_press_callback=self.__on_play_button_press)
-            buttons.append(button)
-            if self.__album.storage_type & StorageType.EPHEMERAL:
-                button = Gtk.Button.new_from_icon_name(
-                    "document-save-symbolic", Gtk.IconSize.MENU)
-                button.set_tooltip_text(_("Save in collection"))
-                self.__gesture_save = GesturesHelper(
-                    button,
-                    primary_press_callback=self.__on_save_button_press)
-                buttons.append(button)
-        if self.__view_type & (ViewType.PLAYBACK |
-                               ViewType.PLAYLISTS |
-                               ViewType.SEARCH):
-            button = Gtk.Button()
-            image = Gtk.Image.new()
-            button.set_image(image)
-            self.__update_list_button(button)
-            self.__gesture_list = GesturesHelper(
-                button,
-                primary_press_callback=self.__on_list_button_press)
-            buttons.append(button)
-
+            button.connect("clicked", self.__on_play_clicked)
+        button.set_relief(Gtk.ReliefStyle.NONE)
+        button.get_style_context().add_class("menu-button")
+        button.set_property("valign", Gtk.Align.CENTER)
         header = Gtk.Grid.new()
         header.set_column_spacing(MARGIN_SMALL)
         header.show()
         header.set_margin_start(MARGIN_SMALL)
-        # 2px for trackrow padding
-        header.set_margin_end(MARGIN_SMALL + 2)
+        # This to align button with row button
+        header.set_margin_end(MARGIN_SMALL * 2 + 3)
         header.set_margin_top(2)
         header.set_margin_bottom(2)
         header.attach(self.__artwork, 0, 0, 1, 2)
         header.attach(self.__artist_label, 1, 0, 1, 1)
         header.attach(self.__title_label, 1, 1, 1, 1)
-        i = 2
-        for button in buttons:
-            button.set_relief(Gtk.ReliefStyle.NONE)
-            button.get_style_context().add_class("menu-button")
-            button.set_property("valign", Gtk.Align.CENTER)
-            header.attach(button, i, 0, 1, 2)
-            i += 1
+        header.attach(button, 2, 0, 1, 2)
 
         self.__revealer = Gtk.Revealer.new()
         self.__revealer.show()
@@ -170,8 +141,8 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         box.pack_start(header, 0, True, True)
         box.pack_start(self.__revealer, 1, False, False)
         self.add(box)
-        self.set_playing_indicator()
         self.set_artwork()
+        self.set_selection()
 
     def reveal(self, reveal=None):
         """
@@ -192,7 +163,7 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
             self.get_style_context().remove_class("albumrow-collapsed")
             self.unset_state_flags(Gtk.StateFlags.VISITED)
 
-    def set_playing_indicator(self):
+    def set_selection(self):
         """
             Show play indicator
         """
@@ -201,20 +172,22 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         selected = self.album.id == App().player.current_track.album.id and\
             App().player.current_track.id in self.album.track_ids
         if self.__revealer.get_reveal_child():
-            self.__tracks_view.set_playing_indicator()
             self.set_state_flags(Gtk.StateFlags.NORMAL, True)
         elif selected:
             self.set_state_flags(Gtk.StateFlags.VISITED, True)
         else:
             self.set_state_flags(Gtk.StateFlags.NORMAL, True)
 
-    def stop(self):
+    def reset(self):
         """
-            Stop view loading
+            Get a new track view
         """
-        self.__artwork = None
-        if self.__tracks_view.is_populated:
-            self.__tracks_view.stop()
+        if self.__artwork is None:
+            return
+        self.__tracks_view.destroy()
+        self.__tracks_view = self.__get_new_tracks_view()
+        self.__tracks_view.populate()
+        self.__revealer.add(self.__tracks_view)
 
     def set_artwork(self):
         """
@@ -293,59 +266,29 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
     def album(self):
         """
             Get album
-            @return row id as int
+            @return Album
         """
         return self.__album
 
-#######################
-# PROTECTED           #
-#######################
-    def _on_playback_changed(self, player):
-        """
-            Update button state
-        """
-        if self.__artwork is not None and self.__gesture_list is not None:
-            self.__update_list_button(self.__gesture_list.widget)
-
 #######################
 # PRIVATE             #
 #######################
-    def __update_list_button(self, button):
+    def __get_new_tracks_view(self):
         """
-            Update list button based on current status
-            @param button as Gtk.Button
+            Get a new track view
+            @return AlbumTracksView
         """
-        if self.__view_type & ViewType.SEARCH and\
-                self.__album.id not in App().player.album_ids:
-            button.get_image().set_from_icon_name("list-add-symbolic",
-                                                  Gtk.IconSize.MENU)
-            button.set_tooltip_text(_("Add to playback"))
-        else:
-            button.get_image().set_from_icon_name("list-remove-symbolic",
-                                                  Gtk.IconSize.MENU)
-            if self.__view_type & ViewType.PLAYLISTS:
-                button.set_tooltip_text(_("Remove from playlist"))
-            else:
-                button.set_tooltip_text(_("Remove from playback"))
-
-    def __popup_menu(self, widget):
-        """
-            Popup menu for album
-            @param widget as Gtk.Widget
-        """
-        def on_hidden(popover, hide):
-            self.unset_state_flags(Gtk.StateFlags.CHECKED)
-
-        from lollypop.menu_objects import AlbumMenu
-        from lollypop.widgets_menu import MenuBuilder
-        menu = AlbumMenu(self.__album, ViewType.ALBUM,
-                         App().window.is_adaptive)
-        menu_widget = MenuBuilder(menu)
-        menu_widget.show()
-        self.set_state_flags(Gtk.StateFlags.CHECKED, True)
-        popover = popup_widget(menu_widget, widget)
-        if popover is not None:
-            popover.connect("hidden", on_hidden)
+        tracks_view = AlbumTracksView(self.__album,
+                                      self.__view_type |
+                                      ViewType.SINGLE_COLUMN)
+        tracks_view.connect("activated",
+                            self.__on_tracks_view_activated)
+        tracks_view.connect("populated",
+                            self.__on_tracks_view_populated)
+        tracks_view.connect("track-removed",
+                            self.__on_tracks_view_track_removed)
+        tracks_view.show()
+        return tracks_view
 
     def __on_album_artwork(self, surface):
         """
@@ -359,7 +302,6 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
                                               Gtk.IconSize.BUTTON)
         else:
             self.__artwork.set_from_surface(surface)
-            del surface
         self.show_all()
         # TracksView will emit populated
         if not self.revealed:
@@ -379,9 +321,7 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         if layout_title.is_ellipsized() or layout_artist.is_ellipsized():
             artist = GLib.markup_escape_text(self.__artist_label.get_text())
             title = GLib.markup_escape_text(self.__title_label.get_text())
-            # Workaround a recent issue in GTK+ 3.24.11
-            # self.set_tooltip_markup("<b>%s</b>\n%s" % (artist, title))
-            self.set_tooltip_markup("<b>%s</b> - %s" % (artist, title))
+            self.set_tooltip_markup("<b>%s</b>\n%s" % (artist, title))
         else:
             self.set_tooltip_text("")
 
@@ -406,9 +346,7 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
             @param row as TrackRow
         """
         row.destroy()
-        App().player.remove_track_from_album(row.track, row.track.album)
-        if len(self.children) == 0:
-            self.destroy()
+        emit_signal(self, "track-removed", row.track)
 
     def __on_tracks_view_populated(self, view):
         """
@@ -421,75 +359,20 @@ class AlbumRow(Gtk.ListBoxRow, SignalsHelper):
         else:
             emit_signal(self, "populated")
 
-    def __on_list_button_press(self, x, y, event):
+    def __on_remove_clicked(self, button):
         """
-            Add/Remove album from playlist/playback
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
+            Remove album from playback/playlist
+            @param button as Gtk.Button
         """
         if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
             return True
-        if self.__view_type & (ViewType.PLAYBACK | ViewType.SEARCH):
-            if self.__album.id in App().player.album_ids:
-                App().player.remove_album(self.__album)
-            else:
-                App().player.add_album(self.__album)
-            if self.__view_type & ViewType.PLAYBACK:
-                self.destroy()
-            else:
-                self.__update_list_button(self.__gesture_list.widget)
-        elif self.__view_type & ViewType.PLAYLISTS:
-            if App().player.current_track.album.id == self.__album.id:
-                # Stop playback or loop for last album
-                # Else skip current
-                if len(App().player.albums) == 1:
-                    App().player.remove_album(self.__album)
-                    App().player.next()
-                else:
-                    App().player.skip_album()
-                    App().player.remove_album(self.__album)
-            else:
-                App().player.remove_album(self.__album)
-            from lollypop.view_playlists import PlaylistsView
-            view = self.get_ancestor(PlaylistsView)
-            if view is not None:
-                view.remove_from_playlist(self.__album)
-            self.destroy()
+        self.destroy()
 
-    def __on_save_button_press(self, x, y, event):
+    def __on_play_clicked(self, button):
         """
-            Show row menu
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
-        """
-        if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
-            return True
-        if self.__album.storage_type & StorageType.EPHEMERAL:
-            for artist in self.__album.artists:
-                App().art.cache_artist_artwork(artist)
-            self.__album.save(True)
-            self.__gesture_save.widget.set_sensitive(False)
-
-    def __on_play_button_press(self, x, y, event):
-        """
-            Show row menu
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
+            Play album
+            @param button as Gtk.Button
         """
         if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
             return True
         App().player.play_album(self.__album)
-
-    def __on_menu_button_press(self, x, y, event):
-        """
-            Show row menu
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
-        """
-        if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
-            return True
-        self.__popup_menu(self.__gesture_menu.widget)
