@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@ from gi.repository import Gtk, Pango, GLib, GObject
 from gettext import gettext as _
 
 from lollypop.define import App, ViewType, MARGIN_SMALL, IndicatorType
-from lollypop.define import StorageType
+from lollypop.define import StorageType, LovedFlags
 from lollypop.widgets_indicator import IndicatorWidget
 from lollypop.utils import ms_to_string, on_query_tooltip, popup_widget
 from lollypop.utils import emit_signal
@@ -47,36 +47,34 @@ class TrackRow(Gtk.ListBoxRow):
             height = min_height
         return height
 
-    def __init__(self, track, album_artist_ids, view_type, show_track_number):
+    def __init__(self, track, album_artist_ids, view_type):
         """
             Init row widgets
             @param track as Track
             @param album_artist_ids as [int]
             @param view_type as ViewType
-            @param show_track_number as bool
         """
         # We do not use Gtk.Builder for speed reasons
         Gtk.ListBoxRow.__init__(self)
         self.__view_type = view_type
-        self.__show_track_number = show_track_number
         self._track = track
+        self.set_property("margin", MARGIN_SMALL)
         self._grid = Gtk.Grid()
         self._grid.set_property("valign", Gtk.Align.CENTER)
         self._grid.set_column_spacing(5)
         self._grid.show()
-        self._indicator = IndicatorWidget(self, view_type)
-        self._indicator.show()
-        self._grid.add(self._indicator)
+        if not view_type & ViewType.QUEUE:
+            self._indicator = IndicatorWidget(self, view_type)
+            self._indicator.show()
+            self._indicator.set_valign(Gtk.Align.CENTER)
+            self._grid.add(self._indicator)
         self._num_label = Gtk.Label.new()
         self._num_label.set_ellipsize(Pango.EllipsizeMode.END)
         self._num_label.set_width_chars(4)
         self._num_label.get_style_context().add_class("dim-label")
-        self._num_label.show()
         self.update_number_label()
         self._grid.add(self._num_label)
-        self.__title_label = Gtk.Label.new(
-            GLib.markup_escape_text(self._track.name))
-        self.__title_label.set_use_markup(True)
+        self.__title_label = Gtk.Label.new()
         self.__title_label.set_property("has-tooltip", True)
         self.__title_label.connect("query-tooltip", on_query_tooltip)
         self.__title_label.set_property("hexpand", True)
@@ -86,47 +84,45 @@ class TrackRow(Gtk.ListBoxRow):
         self.__title_label.show()
         self._grid.add(self.__title_label)
         featuring_artist_ids = track.get_featuring_artist_ids(album_artist_ids)
+        markup = GLib.markup_escape_text(self._track.name)
         if featuring_artist_ids:
             artists = []
             for artist_id in featuring_artist_ids:
                 artists.append(App().artists.get_name(artist_id))
-            artists_label = Gtk.Label.new(GLib.markup_escape_text(
-                ", ".join(artists)))
-            artists_label.set_use_markup(True)
-            artists_label.set_property("has-tooltip", True)
-            artists_label.connect("query-tooltip", on_query_tooltip)
-            artists_label.set_property("hexpand", True)
-            artists_label.set_property("halign", Gtk.Align.END)
-            artists_label.set_ellipsize(Pango.EllipsizeMode.END)
-            artists_label.set_opacity(0.3)
-            artists_label.set_margin_end(5)
-            artists_label.show()
-            self._grid.add(artists_label)
+            markup += "\n<span size='x-small' alpha='40000'>%s</span>" %\
+                GLib.markup_escape_text(", ".join(artists))
+        self.__title_label.set_markup(markup)
         duration = ms_to_string(self._track.duration)
         self.__duration_label = Gtk.Label.new(duration)
         self.__duration_label.get_style_context().add_class("dim-label")
         self.__duration_label.show()
         self._grid.add(self.__duration_label)
-        if self.__view_type & (ViewType.PLAYBACK | ViewType.PLAYLISTS):
+        self.__action_button = None
+        if self.__view_type & (
+                ViewType.PLAYBACK | ViewType.PLAYLISTS):
             self.__action_button = Gtk.Button.new_from_icon_name(
                "list-remove-symbolic",
                Gtk.IconSize.MENU)
             self.__action_button.set_tooltip_text(
                _("Remove from playlist"))
-        else:
+        elif self.__view_type & (ViewType.ALBUM | ViewType.ARTIST):
             self.__action_button = Gtk.Button.new_from_icon_name(
                "view-more-symbolic",
                Gtk.IconSize.MENU)
-        self.__action_button.show()
-        self.__action_button.connect("clicked",
-                                     self.__on_action_button_clicked)
-        self.__action_button.set_margin_end(MARGIN_SMALL)
-        self.__action_button.set_relief(Gtk.ReliefStyle.NONE)
-        context = self.__action_button.get_style_context()
-        context.add_class("menu-button")
-        self._grid.add(self.__action_button)
+        if self.__action_button is None:
+            self.__duration_label.set_margin_end(MARGIN_SMALL)
+        else:
+            self.__action_button.show()
+            self.__action_button.connect("clicked",
+                                         self.__on_action_button_clicked)
+            self.__action_button.set_margin_end(MARGIN_SMALL)
+            self.__action_button.set_relief(Gtk.ReliefStyle.NONE)
+            context = self.__action_button.get_style_context()
+            context.add_class("menu-button")
+            self._grid.add(self.__action_button)
         self.add(self._grid)
-        self.set_indicator(self._get_indicator_type())
+        if not view_type & ViewType.QUEUE:
+            self.set_indicator(self._get_indicator_type())
         self.update_duration()
         self.get_style_context().add_class("trackrow")
 
@@ -161,7 +157,7 @@ class TrackRow(Gtk.ListBoxRow):
             if indicator_type & IndicatorType.LOVED:
                 self._indicator.set_opacity(1)
                 self._indicator.loved()
-            elif indicator_type & IndicatorType.SKIP:
+            elif indicator_type & IndicatorType.SKIPPED:
                 self._indicator.set_opacity(1)
                 self._indicator.skip()
             else:
@@ -176,21 +172,20 @@ class TrackRow(Gtk.ListBoxRow):
             pos = App().player.get_track_position(self._track.id)
             self._num_label.set_text(str(pos))
             self._num_label.show()
-        elif self.__show_track_number:
+        elif not self.__view_type & ViewType.SEARCH:
+            if self.__view_type & (ViewType.PLAYBACK | ViewType.PLAYLISTS):
+                discs = App().albums.get_discs(self._track.album.id)
+                if len(discs) > 1:
+                    label = "%s: %s" % (self._track.discnumber,
+                                        self._track.number)
+                else:
+                    label = "%s" % self._track.number
+            else:
+                label = "%s" % self._track.number
+            self._num_label.set_markup(label)
             self._num_label.get_style_context().remove_class("queued")
-            # Just track number for albums or playlists if only one album
-            if self.__view_type & ViewType.ALBUM or\
-                    len(self._track.album.discs) == 1:
-                self._num_label.set_text(str(self._track.number))
-            # Prepend disc number
-            elif len(self._track.album.discs) > 1:
-                discnumber = App().tracks.get_discnumber(self._track.id)
-                label = "(%s)  %s" % (discnumber, self._track.number)
-                self._num_label.set_text(label)
             self._num_label.show()
         else:
-            self._num_label.get_style_context().remove_class("queued")
-            self._num_label.set_text("")
             self._num_label.hide()
 
     def popup_menu(self, parent, x=None, y=None):
@@ -205,15 +200,17 @@ class TrackRow(Gtk.ListBoxRow):
 
         from lollypop.menu_objects import TrackMenu, TrackMenuExt
         from lollypop.widgets_menu import MenuBuilder
-        menu = TrackMenu(self._track, self.__view_type,
-                         App().window.is_adaptive)
+        menu = TrackMenu(self._track, self.__view_type)
         menu_widget = MenuBuilder(menu)
         menu_widget.show()
         if not self._track.storage_type & StorageType.EPHEMERAL:
             menu_ext = TrackMenuExt(self._track)
             menu_ext.show()
-            menu_widget.append_widget(menu_ext)
-        popover = popup_widget(menu_widget, parent, x, y, self)
+            menu_widget.add_widget(menu_ext)
+        if self.is_selected():
+            popover = popup_widget(menu_widget, parent, x, y, None)
+        else:
+            popover = popup_widget(menu_widget, parent, x, y, self)
         if popover is None:
             menu_widget.connect("hidden", on_hidden)
         else:
@@ -246,10 +243,10 @@ class TrackRow(Gtk.ListBoxRow):
         indicator_type = IndicatorType.NONE
         if App().player.current_track.id == self._track.id:
             indicator_type |= IndicatorType.PLAY
-        if self._track.loved == 1:
+        if self._track.loved & LovedFlags.LOVED:
             indicator_type |= IndicatorType.LOVED
-        elif self._track.loved == -1:
-            indicator_type |= IndicatorType.SKIP
+        elif self._track.loved & LovedFlags.SKIPPED:
+            indicator_type |= IndicatorType.SKIPPED
         return indicator_type
 
 #######################
@@ -260,7 +257,8 @@ class TrackRow(Gtk.ListBoxRow):
             Show row menu
             @param button as Gtk.Button
         """
-        if self.__view_type & (ViewType.PLAYBACK | ViewType.PLAYLISTS):
+        if self.__view_type & (
+                ViewType.PLAYBACK | ViewType.QUEUE | ViewType.PLAYLISTS):
             emit_signal(self, "removed")
         else:
             self.popup_menu(button)
