@@ -52,7 +52,7 @@ class ListenBrainzWebService(GObject.GObject):
             self.__queue = load(
                 open(LOLLYPOP_DATA_PATH + "/%s_queue.bin" % self.__name, "rb"))
         except Exception as e:
-            Logger.info("ListenBrainzWebService::__init__(): %s", e)
+            Logger.info("ListenBrainzWebService::start(): %s", e)
             self.__queue = []
 
     def stop(self):
@@ -75,11 +75,13 @@ class ListenBrainzWebService(GObject.GObject):
             @param track as Track
             @param timestamp as int
         """
+        monitor = Gio.NetworkMonitor.get_default()
         if not App().settings.get_value(
                 "listenbrainz-user-token").get_string():
             return
         elif App().settings.get_value("disable-scrobbling") or\
-                not get_network_available():
+                not get_network_available() or\
+                monitor.get_network_metered():
             self.__queue.append((track, timestamp))
         elif track.id is not None and track.id >= 0:
             App().task_helper.run(self.__listen, track, timestamp)
